"""Classe base para instaladores do Docker."""

import subprocess
import shutil
from abc import ABC, abstractmethod
from typing import List, Optional
from rich import print

from ..system_detector import SystemInfo


class BaseInstaller(ABC):
    """Classe base para instaladores do Docker."""
    
    def __init__(self, system_info: SystemInfo):
        """
        Inicializa o instalador.
        
        Args:
            system_info: Informações do sistema
        """
        self.system_info = system_info
    
    @abstractmethod
    def install(self) -> bool:
        """
        Instala o Docker no sistema.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        pass
    
    @abstractmethod
    def uninstall(self) -> bool:
        """
        Remove o Docker do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        pass
    
    @abstractmethod
    def get_install_commands(self) -> List[str]:
        """
        Retorna lista de comandos para instalação manual.
        
        Returns:
            List[str]: Lista de comandos
        """
        pass
    
    def is_docker_installed(self) -> bool:
        """
        Verifica se o Docker está instalado.
        
        Returns:
            bool: True se o Docker está instalado
        """
        return shutil.which("docker") is not None
    
    def get_docker_version(self) -> Optional[str]:
        """
        Obtém a versão do Docker instalada.
        
        Returns:
            Optional[str]: Versão do Docker ou None se não instalado
        """
        try:
            result = subprocess.run(
                ["docker", "--version"], 
                capture_output=True, 
                text=True,
                timeout=10
            )
            if result.returncode == 0:
                # Output típico: "Docker version 20.10.7, build f0df350"
                version_line = result.stdout.strip()
                if "version" in version_line:
                    return version_line.split()[2].rstrip(",")
            return None
        except Exception:
            return None
    
    def test_docker_installation(self) -> bool:
        """
        Testa se o Docker está funcionando corretamente.
        
        Returns:
            bool: True se o Docker está funcionando
        """
        try:
            print("  [blue]🧪[/blue] Testando instalação do Docker...")
            
            # Testar comando docker version
            result = subprocess.run(
                ["docker", "version"],
                capture_output=True,
                text=True,
                timeout=30
            )
            
            if result.returncode != 0:
                # Verificar se é problema de permissão específico
                if "permission denied" in result.stderr.lower() or "connect: permission denied" in result.stderr.lower():
                    print("  [red]✗[/red] Problema de permissão detectado no Docker daemon")
                else:
                    print("  [yellow]![/yellow] Docker instalado mas não está rodando")
                return False
            
            # Tentar rodar container de teste
            print("  [blue]🧪[/blue] Executando container de teste...")
            result = subprocess.run(
                ["docker", "run", "--rm", "hello-world"],
                capture_output=True,
                text=True,
                timeout=60
            )
            
            if result.returncode == 0:
                print("  [green]✓[/green] Docker está funcionando corretamente!")
                return True
            else:
                # Verificar se é problema de permissão específico
                if "permission denied" in result.stderr.lower() or "connect: permission denied" in result.stderr.lower():
                    print("  [red]✗[/red] Problema de permissão detectado ao executar containers")
                else:
                    print("  [yellow]![/yellow] Docker instalado mas não consegue executar containers")
                return False
                
        except subprocess.TimeoutExpired:
            print("  [yellow]![/yellow] Timeout ao testar Docker")
            return False
        except Exception as e:
            print(f"  [yellow]![/yellow] Erro ao testar Docker: {e}")
            return False
    
    def _run_command(self, command: List[str], shell: bool = False, ignore_errors: bool = False) -> subprocess.CompletedProcess:
        """
        Executa um comando no sistema.
        
        Args:
            command: Comando a ser executado
            shell: Se deve usar shell
            ignore_errors: Se deve ignorar erros
            
        Returns:
            subprocess.CompletedProcess: Resultado do comando
            
        Raises:
            subprocess.CalledProcessError: Se o comando falhar e ignore_errors=False
        """
        try:
            if shell:
                cmd_str = " ".join(command)
                result = subprocess.run(
                    cmd_str,
                    shell=True,
                    check=not ignore_errors,
                    capture_output=True,
                    text=True,
                    timeout=300  # 5 minutos
                )
            else:
                result = subprocess.run(
                    command,
                    check=not ignore_errors,
                    capture_output=True,
                    text=True,
                    timeout=300  # 5 minutos
                )
            
            return result
            
        except subprocess.TimeoutExpired:
            raise Exception(f"Comando demorou muito para executar: {' '.join(command)}")
        except subprocess.CalledProcessError as e:
            if not ignore_errors:
                raise e
            return e
    
    def check_prerequisites(self) -> bool:
        """
        Verifica pré-requisitos para instalação.
        
        Returns:
            bool: True se todos os pré-requisitos estão atendidos
        """
        # Verificar se tem sudo (para Linux)
        if self.system_info.os_type.value != "macos":
            if not shutil.which("sudo"):
                print("  [red]✗[/red] sudo não encontrado. Necessário para instalação.")
                return False
        
        # Verificar se tem curl
        if not shutil.which("curl"):
            print("  [yellow]![/yellow] curl não encontrado. Tentando instalar...")
            try:
                if self.system_info.os_type.value in ["ubuntu", "debian", "wsl_ubuntu", "wsl_debian"]:
                    self._run_command(["sudo", "apt", "install", "-y", "curl"])
                elif self.system_info.os_type.value == "macos":
                    print("  [yellow]![/yellow] Instale curl usando: brew install curl")
                    return False
            except Exception:
                print("  [red]✗[/red] Não foi possível instalar curl.")
                return False
        
        return True
    
    def print_manual_instructions(self) -> None:
        """Imprime instruções para instalação manual."""
        print(f"\n[bold yellow]📋 Instruções para instalação manual no {self.system_info.os_type.value}:[/bold yellow]")
        print()
        
        for i, command in enumerate(self.get_install_commands(), 1):
            print(f"  [blue]{i}.[/blue] {command}")
        
        print()
        print("[bold]💡 Dica:[/bold] Copie e cole os comandos acima no terminal.")
        print("[bold]⚠ Importante:[/bold] Faça logout/login após a instalação para aplicar as permissões.")


"""Módulo para detectar sistema operacional e arquitetura."""

import platform
import subprocess
import os
from typing import Dict, Optional
from enum import Enum


class OperatingSystem(str, Enum):
    """Sistemas operacionais suportados."""
    UBUNTU = "ubuntu"
    DEBIAN = "debian" 
    CENTOS = "centos"
    FEDORA = "fedora"
    RHEL = "rhel"
    MACOS = "macos"
    WINDOWS = "windows"
    WSL_UBUNTU = "wsl_ubuntu"
    WSL_DEBIAN = "wsl_debian"
    UNKNOWN = "unknown"


class Architecture(str, Enum):
    """Arquiteturas suportadas."""
    X86_64 = "x86_64"
    ARM64 = "arm64"
    AARCH64 = "aarch64"
    UNKNOWN = "unknown"


class SystemInfo:
    """Informações do sistema."""
    
    def __init__(self, os_type: OperatingSystem, architecture: Architecture, 
                 is_wsl: bool = False, distro_version: Optional[str] = None):
        self.os_type = os_type
        self.architecture = architecture
        self.is_wsl = is_wsl
        self.distro_version = distro_version
    
    def __str__(self):
        wsl_str = " (WSL)" if self.is_wsl else ""
        version_str = f" {self.distro_version}" if self.distro_version else ""
        return f"{self.os_type.value}{version_str} {self.architecture.value}{wsl_str}"


class SystemDetector:
    """Detector de sistema operacional e arquitetura."""
    
    @staticmethod
    def detect() -> SystemInfo:
        """
        Detecta o sistema operacional atual.
        
        Returns:
            SystemInfo com informações do sistema
        """
        # Detectar arquitetura
        arch = SystemDetector._detect_architecture()
        
        # Detectar se está no WSL
        is_wsl = SystemDetector._is_wsl()
        
        # Detectar sistema operacional
        system = platform.system().lower()
        
        if system == "linux":
            return SystemDetector._detect_linux_distro(arch, is_wsl)
        elif system == "darwin":
            return SystemDetector._detect_macos(arch)
        elif system == "windows":
            return SystemInfo(OperatingSystem.WINDOWS, arch)
        else:
            return SystemInfo(OperatingSystem.UNKNOWN, arch)
    
    @staticmethod
    def _detect_architecture() -> Architecture:
        """Detecta a arquitetura do processador."""
        machine = platform.machine().lower()
        
        if machine in ["x86_64", "amd64"]:
            return Architecture.X86_64
        elif machine in ["arm64", "aarch64"]:
            return Architecture.ARM64
        else:
            return Architecture.UNKNOWN
    
    @staticmethod
    def _is_wsl() -> bool:
        """Verifica se está rodando no WSL."""
        try:
            # Verificar se existe /proc/version com Microsoft
            if os.path.exists("/proc/version"):
                with open("/proc/version", "r") as f:
                    content = f.read().lower()
                    return "microsoft" in content or "wsl" in content
            return False
        except:
            return False
    
    @staticmethod
    def _detect_linux_distro(arch: Architecture, is_wsl: bool) -> SystemInfo:
        """Detecta a distribuição Linux."""
        distro_info = SystemDetector._get_distro_info()
        distro_name = distro_info.get("name", "").lower()
        version = distro_info.get("version", "")
        
        # Ubuntu
        if "ubuntu" in distro_name:
            os_type = OperatingSystem.WSL_UBUNTU if is_wsl else OperatingSystem.UBUNTU
            return SystemInfo(os_type, arch, is_wsl, version)
        
        # Debian
        elif "debian" in distro_name:
            os_type = OperatingSystem.WSL_DEBIAN if is_wsl else OperatingSystem.DEBIAN
            return SystemInfo(os_type, arch, is_wsl, version)
        
        # CentOS
        elif "centos" in distro_name:
            return SystemInfo(OperatingSystem.CENTOS, arch, is_wsl, version)
        
        # Fedora
        elif "fedora" in distro_name:
            return SystemInfo(OperatingSystem.FEDORA, arch, is_wsl, version)
        
        # RHEL
        elif "red hat" in distro_name or "rhel" in distro_name:
            return SystemInfo(OperatingSystem.RHEL, arch, is_wsl, version)
        
        else:
            return SystemInfo(OperatingSystem.UNKNOWN, arch, is_wsl, version)
    
    @staticmethod
    def _detect_macos(arch: Architecture) -> SystemInfo:
        """Detecta versão do macOS."""
        try:
            version = platform.mac_ver()[0]
            return SystemInfo(OperatingSystem.MACOS, arch, False, version)
        except:
            return SystemInfo(OperatingSystem.MACOS, arch)
    
    @staticmethod
    def _get_distro_info() -> Dict[str, str]:
        """Obtém informações da distribuição Linux."""
        info = {}
        
        # Tentar /etc/os-release primeiro
        try:
            with open("/etc/os-release", "r") as f:
                for line in f:
                    if "=" in line:
                        key, value = line.strip().split("=", 1)
                        value = value.strip('"')
                        if key == "NAME":
                            info["name"] = value
                        elif key == "VERSION":
                            info["version"] = value
        except:
            pass
        
        # Fallback para /etc/lsb-release
        if not info:
            try:
                with open("/etc/lsb-release", "r") as f:
                    for line in f:
                        if "=" in line:
                            key, value = line.strip().split("=", 1)
                            value = value.strip('"')
                            if key == "DISTRIB_ID":
                                info["name"] = value
                            elif key == "DISTRIB_RELEASE":
                                info["version"] = value
            except:
                pass
        
        # Fallback para uname
        if not info:
            try:
                result = subprocess.run(["uname", "-a"], capture_output=True, text=True)
                if result.returncode == 0:
                    uname_output = result.stdout.lower()
                    if "ubuntu" in uname_output:
                        info["name"] = "Ubuntu"
                    elif "debian" in uname_output:
                        info["name"] = "Debian"
            except:
                pass
        
        return info
    
    @staticmethod
    def get_package_manager(os_type: OperatingSystem) -> Optional[str]:
        """Retorna o gerenciador de pacotes para o sistema."""
        package_managers = {
            OperatingSystem.UBUNTU: "apt",
            OperatingSystem.WSL_UBUNTU: "apt",
            OperatingSystem.DEBIAN: "apt", 
            OperatingSystem.WSL_DEBIAN: "apt",
            OperatingSystem.CENTOS: "yum",
            OperatingSystem.FEDORA: "dnf",
            OperatingSystem.RHEL: "yum",
            OperatingSystem.MACOS: "brew"
        }
        return package_managers.get(os_type)
    
    @staticmethod
    def supports_docker_installation(os_type: OperatingSystem) -> bool:
        """Verifica se o sistema suporta instalação automática do Docker."""
        supported_systems = [
            OperatingSystem.UBUNTU,
            OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN,
            OperatingSystem.WSL_DEBIAN,
            OperatingSystem.CENTOS,
            OperatingSystem.FEDORA,
            OperatingSystem.RHEL,
            OperatingSystem.MACOS
        ]
        return os_type in supported_systems


"""Gerenciador de ambiente DevOps - Setup de todas as ferramentas necessárias."""

import subprocess
from typing import Dict, List, Optional, Tuple
from dataclasses import dataclass
from rich import print
from rich.console import Console
from rich.table import Table
from rich.progress import Progress, TaskID

from .system_detector import SystemDetector, SystemInfo
from ..config.constants import Tool, DEVOPS_TOOLS_CONFIG


@dataclass
class ToolStatus:
    """Status de uma ferramenta."""
    tool: Tool
    installed: bool
    version: Optional[str] = None
    error: Optional[str] = None


class EnvironmentManager:
    """Gerenciador principal para setup do ambiente DevOps."""
    
    def __init__(self):
        """Inicializa o gerenciador de ambiente."""
        self.console = Console()
        self.system_info = SystemDetector.detect()
        self.tools_status: Dict[Tool, ToolStatus] = {}
    
    def check_tool(self, tool: Tool) -> ToolStatus:
        """
        Verifica se uma ferramenta está instalada.
        
        Args:
            tool: A ferramenta a ser verificada
            
        Returns:
            ToolStatus: Status da ferramenta
        """
        config = DEVOPS_TOOLS_CONFIG[tool]
        
        try:
            result = subprocess.run(
                config["check_command"],
                capture_output=True,
                text=True,
                timeout=10
            )
            
            if result.returncode == 0:
                # Extrair versão do output
                version = self._extract_version(tool, result.stdout)
                return ToolStatus(tool=tool, installed=True, version=version)
            else:
                return ToolStatus(tool=tool, installed=False, error=result.stderr.strip())
                
        except subprocess.TimeoutExpired:
            return ToolStatus(tool=tool, installed=False, error="Timeout")
        except FileNotFoundError:
            return ToolStatus(tool=tool, installed=False, error="Command not found")
        except Exception as e:
            return ToolStatus(tool=tool, installed=False, error=str(e))
    
    def _extract_version(self, tool: Tool, output: str) -> Optional[str]:
        """
        Extrai a versão do output do comando.
        
        Args:
            tool: A ferramenta
            output: Output do comando
            
        Returns:
            str: Versão extraída ou None
        """
        lines = output.strip().split('\n')
        if not lines:
            return None
            
        first_line = lines[0].strip()
        
        # Padrões específicos para cada ferramenta
        if tool == Tool.DOCKER:
            # Docker version 24.0.6, build ed223bc
            if "Docker version" in first_line:
                return first_line.split(',')[0].replace("Docker version", "").strip()
        
        elif tool == Tool.GIT:
            # git version 2.39.2
            if "git version" in first_line:
                return first_line.replace("git version", "").strip()
        
        elif tool == Tool.TERRAFORM:
            # Terraform v1.5.7
            if "Terraform" in first_line:
                return first_line.replace("Terraform", "").strip()
        
        elif tool == Tool.AZURE_CLI:
            # Para az, a versão pode estar em JSON ou texto
            try:
                import json
                data = json.loads(output)
                # Tentar diferentes chaves possíveis
                version = data.get("azure-cli-core") or data.get("azure-cli") or data.get("core")
                if version:
                    return version
            except:
                # Se não for JSON, tentar extrair da primeira linha
                # azure-cli                         2.75.0
                if "azure-cli" in first_line:
                    parts = first_line.split()
                    if len(parts) >= 2:
                        return parts[1]
        
        elif tool == Tool.AWS_CLI:
            # aws-cli/2.13.25 Python/3.11.5
            if "aws-cli" in first_line:
                return first_line.split()[0].replace("aws-cli/", "")
        
        elif tool == Tool.KUBECTL:
            # Client Version: version.Info{Major:"1", Minor:"28"...
            if "Client Version" in first_line:
                # Extrair versão básica
                if "Major:" in first_line and "Minor:" in first_line:
                    import re
                    major = re.search(r'Major:"([^"]+)"', first_line)
                    minor = re.search(r'Minor:"([^"]+)"', first_line)
                    if major and minor:
                        return f"v{major.group(1)}.{minor.group(1)}"
        
        elif tool == Tool.ANSIBLE:
            # ansible [core 2.15.3]
            if "ansible" in first_line:
                import re
                version_match = re.search(r'\[core ([^\]]+)\]', first_line)
                if version_match:
                    return version_match.group(1)
        
        elif tool == Tool.WATCH:
            # watch from procps-ng 3.3.17
            if "watch from" in first_line:
                return first_line.split()[-1]
        
        return first_line
    
    def check_all_tools(self) -> Dict[Tool, ToolStatus]:
        """
        Verifica o status de todas as ferramentas.
        
        Returns:
            Dict[Tool, ToolStatus]: Status de todas as ferramentas
        """
        print("\n:mag: [bold blue]Verificando ferramentas instaladas...[/bold blue]")
        
        with Progress() as progress:
            task = progress.add_task("[blue]Verificando...", total=len(Tool))
            
            for tool in Tool:
                self.tools_status[tool] = self.check_tool(tool)
                progress.update(task, advance=1)
        
        return self.tools_status
    
    def show_status_report(self) -> None:
        """Exibe um relatório detalhado do status das ferramentas."""
        if not self.tools_status:
            self.check_all_tools()
        
        print("\n:clipboard: [bold cyan]Relatório do Ambiente DevOps[/bold cyan]")
        
        # Criar tabela
        table = Table(show_header=True, header_style="bold magenta")
        table.add_column("Ferramenta", style="dim", width=12)
        table.add_column("Status", justify="center", width=10)
        table.add_column("Versão", width=20)
        table.add_column("Descrição", width=30)
        table.add_column("Obrigatória", justify="center", width=10)
        
        # Ordenar por prioridade
        sorted_tools = sorted(Tool, key=lambda t: DEVOPS_TOOLS_CONFIG[t]["priority"])
        
        for tool in sorted_tools:
            config = DEVOPS_TOOLS_CONFIG[tool]
            status = self.tools_status.get(tool)
            
            if not status:
                continue
            
            # Status visual
            if status.installed:
                status_text = "[green]✓ Instalado[/green]"
                version_text = status.version or "Versão não detectada"
            else:
                status_text = "[red]✗ Não instalado[/red]"
                version_text = "[dim]N/A[/dim]"
            
            # Obrigatória
            required_text = "[red]Sim[/red]" if config["required"] else "[yellow]Não[/yellow]"
            
            table.add_row(
                config["name"],
                status_text,
                version_text,
                config["description"],
                required_text
            )
        
        self.console.print(table)
        
        # Resumo
        installed_count = sum(1 for status in self.tools_status.values() if status.installed)
        total_count = len(self.tools_status)
        required_tools = [t for t in Tool if DEVOPS_TOOLS_CONFIG[t]["required"]]
        required_installed = sum(1 for t in required_tools if self.tools_status.get(t, ToolStatus(t, False)).installed)
        
        print(f"\n:chart_with_upwards_trend: [bold]Resumo:[/bold]")
        print(f"  • [blue]Ferramentas instaladas:[/blue] {installed_count}/{total_count}")
        print(f"  • [red]Ferramentas obrigatórias:[/red] {required_installed}/{len(required_tools)}")
        
        if required_installed == len(required_tools):
            print("  • [green]✓ Ambiente mínimo configurado![/green]")
        else:
            missing_required = [t for t in required_tools if not self.tools_status.get(t, ToolStatus(t, False)).installed]
            print(f"  • [red]✗ Faltam ferramentas obrigatórias: {[DEVOPS_TOOLS_CONFIG[t]['name'] for t in missing_required]}[/red]")
    
    def get_missing_tools(self, only_required: bool = False) -> List[Tool]:
        """
        Retorna lista de ferramentas não instaladas.
        
        Args:
            only_required: Se True, retorna apenas ferramentas obrigatórias
            
        Returns:
            List[Tool]: Lista de ferramentas não instaladas
        """
        if not self.tools_status:
            self.check_all_tools()
        
        missing = []
        for tool, status in self.tools_status.items():
            if not status.installed:
                if only_required:
                    if DEVOPS_TOOLS_CONFIG[tool]["required"]:
                        missing.append(tool)
                else:
                    missing.append(tool)
        
        # Ordenar por prioridade
        return sorted(missing, key=lambda t: DEVOPS_TOOLS_CONFIG[t]["priority"])
    
    def is_environment_ready(self) -> bool:
        """
        Verifica se o ambiente está pronto (todas as ferramentas obrigatórias instaladas).
        
        Returns:
            bool: True se todas as ferramentas obrigatórias estão instaladas
        """
        missing_required = self.get_missing_tools(only_required=True)
        return len(missing_required) == 0


"""Instalador do Docker para CentOS/RHEL/Fedora."""

import subprocess
from typing import List
from rich import print

from .base_installer import BaseInstaller
from ..system_detector import OperatingSystem


class RedHatInstaller(BaseInstaller):
    """Instalador do Docker para CentOS, RHEL e Fedora."""
    
    def install(self) -> bool:
        """
        Instala o Docker no CentOS/RHEL/Fedora.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(f":package: Instalando Docker no {self.system_info.os_type.value}...")
        
        try:
            if self.system_info.os_type == OperatingSystem.FEDORA:
                return self._install_fedora()
            else:
                return self._install_centos_rhel()
                
        except subprocess.CalledProcessError as e:
            print(f"  [red]✗[/red] Erro durante a instalação: {e}")
            return False
        except Exception as e:
            print(f"  [red]✗[/red] Erro inesperado: {e}")
            return False
    
    def _install_fedora(self) -> bool:
        """Instala Docker no Fedora."""
        print("  [blue]1/5[/blue] Removendo versões antigas...")
        self._run_command([
            "sudo", "dnf", "remove", "-y",
            "docker", "docker-client", "docker-client-latest",
            "docker-common", "docker-latest", "docker-latest-logrotate",
            "docker-logrotate", "docker-engine"
        ], ignore_errors=True)
        
        print("  [blue]2/5[/blue] Instalando dependências...")
        self._run_command([
            "sudo", "dnf", "install", "-y",
            "dnf-plugins-core"
        ])
        
        print("  [blue]3/5[/blue] Adicionando repositório do Docker...")
        self._run_command([
            "sudo", "dnf", "config-manager", "--add-repo",
            "https://download.docker.com/linux/fedora/docker-ce.repo"
        ])
        
        print("  [blue]4/5[/blue] Instalando Docker CE...")
        self._run_command([
            "sudo", "dnf", "install", "-y",
            "docker-ce", "docker-ce-cli", "containerd.io"
        ])
        
        print("  [blue]5/5[/blue] Configurando Docker...")
        self._configure_docker_service()
        
        return True
    
    def _install_centos_rhel(self) -> bool:
        """Instala Docker no CentOS/RHEL."""
        print("  [blue]1/5[/blue] Removendo versões antigas...")
        self._run_command([
            "sudo", "yum", "remove", "-y",
            "docker", "docker-client", "docker-client-latest",
            "docker-common", "docker-latest", "docker-latest-logrotate",
            "docker-logrotate", "docker-engine"
        ], ignore_errors=True)
        
        print("  [blue]2/5[/blue] Instalando dependências...")
        self._run_command([
            "sudo", "yum", "install", "-y",
            "yum-utils"
        ])
        
        print("  [blue]3/5[/blue] Adicionando repositório do Docker...")
        repo_url = "https://download.docker.com/linux/centos/docker-ce.repo"
        self._run_command([
            "sudo", "yum-config-manager", "--add-repo", repo_url
        ])
        
        print("  [blue]4/5[/blue] Instalando Docker CE...")
        self._run_command([
            "sudo", "yum", "install", "-y",
            "docker-ce", "docker-ce-cli", "containerd.io"
        ])
        
        print("  [blue]5/5[/blue] Configurando Docker...")
        self._configure_docker_service()
        
        return True
    
    def _configure_docker_service(self) -> None:
        """Configura o serviço Docker."""
        try:
            # Iniciar e habilitar Docker
            self._run_command(["sudo", "systemctl", "start", "docker"])
            self._run_command(["sudo", "systemctl", "enable", "docker"])
            
            # Adicionar usuário ao grupo docker
            import os
            username = os.getenv("USER")
            if username:
                self._run_command(["sudo", "usermod", "-aG", "docker", username])
                print(f"  [green]✓[/green] Usuário {username} adicionado ao grupo docker")
                print("  [yellow]⚠[/yellow] Faça logout/login para aplicar as permissões")
            
        except Exception as e:
            print(f"  [yellow]![/yellow] Aviso: Erro na configuração: {e}")
    
    def uninstall(self) -> bool:
        """
        Remove o Docker do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        try:
            print(":wastebasket: Removendo Docker...")
            
            # Parar serviços
            self._run_command(["sudo", "systemctl", "stop", "docker"], ignore_errors=True)
            self._run_command(["sudo", "systemctl", "disable", "docker"], ignore_errors=True)
            
            # Remover pacotes
            if self.system_info.os_type == OperatingSystem.FEDORA:
                self._run_command([
                    "sudo", "dnf", "remove", "-y",
                    "docker-ce", "docker-ce-cli", "containerd.io"
                ])
            else:
                self._run_command([
                    "sudo", "yum", "remove", "-y",
                    "docker-ce", "docker-ce-cli", "containerd.io"
                ])
            
            print("  [green]✓[/green] Docker removido com sucesso!")
            return True
            
        except Exception as e:
            print(f"  [red]✗[/red] Erro durante a remoção: {e}")
            return False
    
    def get_install_commands(self) -> List[str]:
        """Retorna lista de comandos para instalação manual."""
        if self.system_info.os_type == OperatingSystem.FEDORA:
            return [
                "sudo dnf remove -y docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine",
                "sudo dnf install -y dnf-plugins-core",
                "sudo dnf config-manager --add-repo https://download.docker.com/linux/fedora/docker-ce.repo",
                "sudo dnf install -y docker-ce docker-ce-cli containerd.io",
                "sudo systemctl start docker",
                "sudo systemctl enable docker",
                "sudo usermod -aG docker $USER"
            ]
        else:
            return [
                "sudo yum remove -y docker docker-client docker-client-latest docker-common docker-latest docker-latest-logrotate docker-logrotate docker-engine",
                "sudo yum install -y yum-utils",
                "sudo yum-config-manager --add-repo https://download.docker.com/linux/centos/docker-ce.repo",
                "sudo yum install -y docker-ce docker-ce-cli containerd.io",
                "sudo systemctl start docker",
                "sudo systemctl enable docker",
                "sudo usermod -aG docker $USER"
            ]


"""Instalador do AWS CLI v2 para diferentes sistemas operacionais."""

import subprocess
import os
import tempfile
import zipfile
import shutil
from pathlib import Path
from typing import Optional, List
from rich import print

from .base_installer import BaseInstaller
from ..system_detector import SystemInfo, OperatingSystem


class AwsCliInstaller(BaseInstaller):
    """Instalador especializado para AWS CLI v2."""
    
    def __init__(self, system_info: SystemInfo):
        """
        Inicializa o instalador do AWS CLI v2.
        
        Args:
            system_info: Informações do sistema operacional
        """
        super().__init__(system_info)
        self.tool_name = "AWS CLI v2"
        
    def is_installed(self) -> bool:
        """
        Verifica se o AWS CLI v2 está instalado.
        
        Returns:
            bool: True se o AWS CLI v2 estiver instalado
        """
        try:
            result = subprocess.run(
                ["aws", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            # AWS CLI v2 deve mostrar "aws-cli/2.x.x"
            return result.returncode == 0 and "aws-cli/2." in result.stdout
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return False
    
    def get_installed_version(self) -> Optional[str]:
        """
        Obtém a versão instalada do AWS CLI.
        
        Returns:
            Optional[str]: Versão instalada ou None
        """
        try:
            result = subprocess.run(
                ["aws", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            if result.returncode == 0:
                # aws-cli/2.13.25 Python/3.11.5 Linux/5.4.0-74-generic exe/x86_64.ubuntu.20
                version_line = result.stdout.strip()
                if "aws-cli/" in version_line:
                    return version_line.split()[0].replace("aws-cli/", "")
            return None
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return None
    
    def install(self) -> bool:
        """
        Instala o AWS CLI v2 baseado no sistema operacional.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(f":gear: [blue]Instalando {self.tool_name}...[/blue]")
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                return self._install_macos()
            
            elif self.system_info.os_type in [
                OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
                OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN,
                OperatingSystem.CENTOS, OperatingSystem.RHEL,
                OperatingSystem.FEDORA
            ]:
                return self._install_linux()
            
            else:
                print(f":warning: [yellow]Sistema {self.system_info.os_type.value} não suportado para instalação automática do AWS CLI[/yellow]")
                return self._show_manual_instructions()
        
        except Exception as e:
            print(f":x: [red]Erro durante instalação do AWS CLI: {str(e)}[/red]")
            return False
    
    def _install_macos(self) -> bool:
        """Instala AWS CLI v2 no macOS."""
        print(":apple: [blue]Detectado macOS - usando instalador oficial[/blue]")
        
        try:
            # Determinar arquitetura
            arch = self._get_macos_architecture()
            if not arch:
                print(":x: [red]Não foi possível determinar arquitetura do macOS[/red]")
                return False
            
            url = f"https://awscli.amazonaws.com/AWSCLIV2-{arch}.pkg"
            
            with tempfile.TemporaryDirectory() as temp_dir:
                temp_path = Path(temp_dir)
                pkg_file = temp_path / "AWSCLIV2.pkg"
                
                # Download
                print(f":arrow_down: [blue]Baixando AWS CLI v2 para {arch}...[/blue]")
                result = subprocess.run([
                    "curl", "-L", "-o", str(pkg_file), url
                ], capture_output=True)
                
                if result.returncode != 0:
                    print(":x: [red]Falha no download[/red]")
                    return False
                
                # Instalar
                print(":package: [blue]Instalando AWS CLI v2...[/blue]")
                result = subprocess.run([
                    "sudo", "installer", "-pkg", str(pkg_file), "-target", "/"
                ], capture_output=True)
                
                if result.returncode == 0:
                    print(":white_check_mark: [green]AWS CLI v2 instalado via instalador oficial![/green]")
                    return True
                else:
                    print(f":x: [red]Falha na instalação: {result.stderr.decode()}[/red]")
                    return False
        
        except Exception as e:
            print(f":x: [red]Erro na instalação para macOS: {str(e)}[/red]")
            return False
    
    def _install_linux(self) -> bool:
        """Instala AWS CLI v2 no Linux."""
        print(":penguin: [blue]Detectado Linux - usando instalador oficial[/blue]")
        
        try:
            # Determinar arquitetura
            arch = self._get_linux_architecture()
            if not arch:
                print(":x: [red]Não foi possível determinar arquitetura do Linux[/red]")
                return False
            
            url = f"https://awscli.amazonaws.com/awscli-exe-linux-{arch}.zip"
            
            with tempfile.TemporaryDirectory() as temp_dir:
                temp_path = Path(temp_dir)
                zip_file = temp_path / "awscliv2.zip"
                
                # Download
                print(f":arrow_down: [blue]Baixando AWS CLI v2 para {arch}...[/blue]")
                result = subprocess.run([
                    "curl", "-L", "-o", str(zip_file), url
                ], capture_output=True)
                
                if result.returncode != 0:
                    print(":x: [red]Falha no download[/red]")
                    return False
                
                # Extrair
                print(":package: [blue]Extraindo arquivo...[/blue]")
                with zipfile.ZipFile(zip_file, 'r') as zip_ref:
                    zip_ref.extractall(temp_path)
                
                # Instalar
                install_script = temp_path / "aws" / "install"
                if not install_script.exists():
                    print(":x: [red]Script de instalação não encontrado[/red]")
                    return False
                
                # Tornar script executável
                install_script.chmod(0o755)
                
                print(":gear: [blue]Executando instalador...[/blue]")
                result = subprocess.run([
                    "sudo", str(install_script)
                ], capture_output=True)
                
                if result.returncode == 0:
                    # Corrigir permissões dos binários
                    self._fix_binary_permissions()
                    print(":white_check_mark: [green]AWS CLI v2 instalado via instalador oficial![/green]")
                    return True
                else:
                    # Tentar instalação sem sudo em diretório local
                    print(":information: [blue]Tentando instalação sem sudo...[/blue]")
                    return self._install_linux_local(install_script)
        
        except Exception as e:
            print(f":x: [red]Erro na instalação para Linux: {str(e)}[/red]")
            return False
    
    def _install_linux_local(self, install_script: Path) -> bool:
        """Instala AWS CLI v2 em diretório local no Linux."""
        try:
            # Criar diretório local
            local_dir = Path.home() / ".local"
            local_dir.mkdir(exist_ok=True)
            
            # Tornar script executável
            install_script.chmod(0o755)
            
            result = subprocess.run([
                str(install_script), "--install-dir", str(local_dir / "aws-cli"),
                "--bin-dir", str(local_dir / "bin")
            ], capture_output=True)
            
            if result.returncode == 0:
                print(":white_check_mark: [green]AWS CLI v2 instalado em ~/.local/![/green]")
                print(":information: [blue]Adicione ~/.local/bin ao seu PATH se necessário[/blue]")
                return True
            else:
                print(f":x: [red]Falha na instalação local: {result.stderr.decode()}[/red]")
                return False
        
        except Exception as e:
            print(f":x: [red]Erro na instalação local: {str(e)}[/red]")
            return False
    
    def _get_macos_architecture(self) -> Optional[str]:
        """Retorna a arquitetura para macOS."""
        try:
            result = subprocess.run(["uname", "-m"], capture_output=True, text=True)
            if result.returncode == 0:
                arch = result.stdout.strip()
                if arch in ["arm64", "aarch64"]:
                    return "arm64"
                elif arch in ["x86_64", "amd64"]:
                    return "x86_64"
        except Exception:
            pass
        return None
    
    def _get_linux_architecture(self) -> Optional[str]:
        """Retorna a arquitetura para Linux."""
        try:
            result = subprocess.run(["uname", "-m"], capture_output=True, text=True)
            if result.returncode == 0:
                arch = result.stdout.strip()
                if arch in ["aarch64", "arm64"]:
                    return "aarch64"
                elif arch in ["x86_64", "amd64"]:
                    return "x86_64"
        except Exception:
            pass
        return None
    
    def _fix_binary_permissions(self) -> None:
        """Corrige permissões dos binários do AWS CLI."""
        try:
            binaries = ["/usr/local/bin/aws", "/usr/local/bin/aws_completer"]
            for binary in binaries:
                if Path(binary).exists():
                    subprocess.run(["sudo", "chmod", "755", binary], capture_output=True)
        except Exception:
            pass  # Não é crítico se falhar
    
    def _show_manual_instructions(self) -> bool:
        """Mostra instruções para instalação manual."""
        print(":information_source: [cyan]Instruções para instalação manual do AWS CLI v2:[/cyan]")
        print()
        print("1. Acesse: https://docs.aws.amazon.com/cli/latest/userguide/getting-started-install.html")
        print("2. Baixe o instalador para seu sistema operacional")
        print("3. Execute o instalador seguindo as instruções")
        print("4. Verifique a instalação com: aws --version")
        print()
        return False
    
    def get_install_commands(self) -> List[str]:
        """
        Retorna lista de comandos para instalação manual.
        
        Returns:
            List[str]: Lista de comandos
        """
        if self.system_info.os_type == OperatingSystem.MACOS:
            arch = self._get_macos_architecture() or "x86_64"
            return [
                "# Via instalador oficial AWS",
                f"curl -L -o AWSCLIV2-{arch}.pkg https://awscli.amazonaws.com/AWSCLIV2-{arch}.pkg",
                f"sudo installer -pkg AWSCLIV2-{arch}.pkg -target /",
                "rm AWSCLIV2-{arch}.pkg"
            ]
        
        elif self.system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN,
            OperatingSystem.CENTOS, OperatingSystem.RHEL,
            OperatingSystem.FEDORA
        ]:
            arch = self._get_linux_architecture() or "x86_64"
            return [
                "# Via instalador oficial AWS",
                f"curl -L -o awscliv2.zip https://awscli.amazonaws.com/awscli-exe-linux-{arch}.zip",
                "unzip awscliv2.zip",
                "sudo ./aws/install",
                "rm -rf aws awscliv2.zip"
            ]
        
        else:
            return [
                "# Instalação manual",
                "1. Acesse: https://docs.aws.amazon.com/cli/latest/userguide/getting-started-install.html",
                "2. Baixe o instalador para seu sistema operacional",
                "3. Execute o instalador seguindo as instruções",
                "4. Verifique com: aws --version"
            ]

    def uninstall(self) -> bool:
        """
        Remove o AWS CLI v2 do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        print(f":wastebasket: [blue]Removendo {self.tool_name}...[/blue]")
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                # No macOS, remover via paths padrão
                paths_to_remove = [
                    "/usr/local/bin/aws",
                    "/usr/local/bin/aws_completer",
                    "/usr/local/aws-cli"
                ]
            else:
                # No Linux, remover via paths padrão
                paths_to_remove = [
                    "/usr/local/bin/aws",
                    "/usr/local/bin/aws_completer",
                    "/usr/local/aws-cli"
                ]
            
            removed_any = False
            for path in paths_to_remove:
                if Path(path).exists():
                    try:
                        subprocess.run(["sudo", "rm", "-rf", path], check=True)
                        print(f":white_check_mark: [green]Removido: {path}[/green]")
                        removed_any = True
                    except subprocess.CalledProcessError:
                        continue
            
            if removed_any:
                print(":white_check_mark: [green]AWS CLI v2 removido com sucesso![/green]")
                return True
            else:
                print(":x: [red]AWS CLI v2 não encontrado para remoção[/red]")
                return False
        
        except Exception as e:
            print(f":x: [red]Erro ao remover AWS CLI v2: {str(e)}[/red]")
            return False


"""Comandos para instalação de ferramentas."""

import typer
from rich import print
from typing import Optional

from ..system.docker_installer import DockerInstaller
from ..system.installers.terraform_installer import TerraformInstaller
from ..system.installers.azure_cli_installer import AzureCliInstaller
from ..system.installers.aws_cli_installer import AwsCliInstaller
from ..system.system_detector import SystemDetector


def install_docker(
    check_only: bool = typer.Option(False, "--check-only", help="Apenas verificar se o Docker está instalado"),
    force: bool = typer.Option(False, "--force", "-f", help="Forçar reinstalação mesmo se já estiver instalado"),
    manual: bool = typer.Option(False, "--manual", help="Mostrar instruções para instalação manual"),
    no_test: bool = typer.Option(False, "--no-test", help="Não testar a instalação após completar")
) -> None:
    """
    Instala o Docker automaticamente baseado no sistema operacional detectado.
    
    Sistemas suportados:
    - Ubuntu/Debian (incluindo WSL)
    - macOS (Intel e Apple Silicon)
    - CentOS/RHEL/Fedora
    """
    try:
        docker_installer = DockerInstaller()
        
        # Modo apenas verificação
        if check_only:
            print(":mag: [bold blue]Verificando instalação do Docker...[/bold blue]")
            print()
            docker_installer.print_status()
            return
        
        # Modo instruções manuais
        if manual:
            print(":book: [bold blue]Instruções para instalação manual[/bold blue]")
            print()
            docker_installer.get_manual_instructions()
            return
        
        # Instalação automática
        success = docker_installer.install(
            force=force, 
            test_after_install=not no_test
        )
        
        if success:
            print()
            print(":tada: [bold green]Instalação concluída![/bold green]")
            print()
            print("[bold]Próximos passos:[/bold]")
            print("• Use 'docker --version' para verificar a instalação")
            print("• Use 'docker run hello-world' para testar")
            if docker_installer.system_info.os_type.value != "macos":
                print("• Faça logout/login para aplicar permissões de grupo")
        else:
            raise typer.Exit(code=1)
            
    except Exception as e:
        print(f":x: [bold red]Erro inesperado:[/bold red] {e}")
        raise typer.Exit(code=1)


def uninstall_docker() -> None:
    """
    Remove o Docker do sistema.
    """
    try:
        docker_installer = DockerInstaller()
        
        print(":wastebasket: [bold yellow]Remoção do Docker[/bold yellow]")
        print(f"Sistema: {docker_installer.system_info}")
        print()
        
        success = docker_installer.uninstall()
        
        if success:
            print()
            print(":white_check_mark: [bold green]Docker removido com sucesso![/bold green]")
        else:
            raise typer.Exit(code=1)
            
    except Exception as e:
        print(f":x: [bold red]Erro inesperado:[/bold red] {e}")
        raise typer.Exit(code=1)


def check_docker_status() -> None:
    """
    Verifica o status da instalação do Docker.
    """
    try:
        docker_installer = DockerInstaller()
        
        print(":whale: [bold blue]Status do Docker[/bold blue]")
        print()
        docker_installer.print_status()
        
        # Informações adicionais
        info = docker_installer.check_installation()
        print()
        
        if info['installed'] and info['working']:
            print("[bold green]✓ Docker está pronto para uso![/bold green]")
        elif info['installed'] and not info['working']:
            print("[bold yellow]⚠ Docker instalado mas não está funcionando[/bold yellow]")
            print("Tente:")
            print("• Reiniciar o sistema")
            print("• Iniciar o Docker manualmente")
            if docker_installer.system_info.os_type.value == "macos":
                print("• Abrir Docker Desktop")
        elif info['supported']:
            print("[bold red]✗ Docker não está instalado[/bold red]")
            print(f"Use: [cyan]python3 main.py install docker[/cyan]")
        else:
            print("[bold red]✗ Sistema não suportado[/bold red]")
            print("Visite: https://docs.docker.com/get-docker/")
            
    except Exception as e:
        print(f":x: [bold red]Erro inesperado:[/bold red] {e}")
        raise typer.Exit(code=1)


def system_info() -> None:
    """
    Mostra informações detalhadas do sistema.
    """
    try:
        from ..system.system_detector import SystemDetector
        
        system_info = SystemDetector.detect()
        
        print(":computer: [bold blue]Informações do Sistema[/bold blue]")
        print()
        print(f"[bold]Sistema Operacional:[/bold] {system_info.os_type.value}")
        print(f"[bold]Arquitetura:[/bold] {system_info.architecture.value}")
        print(f"[bold]WSL:[/bold] {'Sim' if system_info.is_wsl else 'Não'}")
        if system_info.distro_version:
            print(f"[bold]Versão:[/bold] {system_info.distro_version}")
        
        print()
        print(f"[bold]Gerenciador de Pacotes:[/bold] {SystemDetector.get_package_manager(system_info.os_type) or 'N/A'}")
        print(f"[bold]Instalação Docker Suportada:[/bold] {'Sim' if SystemDetector.supports_docker_installation(system_info.os_type) else 'Não'}")
        
    except Exception as e:
        print(f":x: [bold red]Erro inesperado:[/bold red] {e}")
        raise typer.Exit(code=1)


def install_azure_cli(force: bool = False, manual: bool = False) -> None:
    """
    Instala o Azure CLI automaticamente baseado no sistema operacional.
    
    Args:
        force: Forçar reinstalação mesmo se já estiver instalado
        manual: Mostrar instruções para instalação manual
    """
    try:
        system_info = SystemDetector.detect()
        azure_installer = AzureCliInstaller(system_info)
        
        print(":cloud: [bold blue]Instalação do Azure CLI[/bold blue]")
        print(f"Sistema detectado: [green]{system_info}[/green]")
        print()
        
        # Mostrar instruções manuais se solicitado
        if manual:
            azure_installer.print_manual_instructions()
            return
        
        # Verificar se já está instalado
        if not force and azure_installer.is_installed():
            version = azure_installer.get_installed_version()
            print(f":white_check_mark: Azure CLI já está instalado (versão {version})")
            
            if not typer.confirm("Deseja reinstalar?"):
                return
        
        # Instalar Azure CLI
        print()
        success = azure_installer.install()
        
        if success:
            print()
            print(":white_check_mark: [bold green]Azure CLI instalado com sucesso![/bold green]")
            print("Teste com: [cyan]az --version[/cyan]")
        else:
            print()
            print(":x: [bold red]Falha na instalação automática.[/bold red]")
            print()
            azure_installer.print_manual_instructions()
            raise typer.Exit(code=1)
            
    except Exception as e:
        print(f":x: [bold red]Erro inesperado:[/bold red] {e}")
        raise typer.Exit(code=1)


def install_terraform(force: bool = False, manual: bool = False) -> None:
    """
    Instala o Terraform automaticamente baseado no sistema operacional.
    
    Args:
        force: Forçar reinstalação mesmo se já estiver instalado
        manual: Mostrar instruções para instalação manual
    """
    try:
        system_info = SystemDetector.detect()
        terraform_installer = TerraformInstaller(system_info)
        
        print(":gear: [bold blue]Instalação do Terraform[/bold blue]")
        print(f"Sistema detectado: [green]{system_info}[/green]")
        print()
        
        # Mostrar instruções manuais se solicitado
        if manual:
            terraform_installer.print_manual_instructions()
            return
        
        # Verificar se já está instalado
        if not force and terraform_installer.is_installed():
            version = terraform_installer.get_installed_version()
            print(f":white_check_mark: Terraform já está instalado (versão {version})")
            
            if not typer.confirm("Deseja reinstalar?"):
                return
        
        # Instalar Terraform
        print()
        success = terraform_installer.install()
        
        if success:
            print()
            print(":white_check_mark: [bold green]Terraform instalado com sucesso![/bold green]")
            print("Teste com: [cyan]terraform --version[/cyan]")
        else:
            print()
            print(":x: [bold red]Falha na instalação automática.[/bold red]")
            print()
            terraform_installer.print_manual_instructions()
            raise typer.Exit(code=1)
            
    except Exception as e:
        print(f":x: [bold red]Erro inesperado:[/bold red] {e}")
        raise typer.Exit(code=1)


def install_aws_cli(force: bool = False, manual: bool = False) -> None:
    """
    Instala o AWS CLI v2 automaticamente baseado no sistema operacional.
    
    Args:
        force: Forçar reinstalação mesmo se já estiver instalado
        manual: Mostrar instruções para instalação manual
    """
    try:
        system_info = SystemDetector.detect()
        aws_installer = AwsCliInstaller(system_info)
        
        print(":cloud: [bold blue]Instalação do AWS CLI v2[/bold blue]")
        print(f"Sistema detectado: [green]{system_info}[/green]")
        print()
        
        # Mostrar instruções manuais se solicitado
        if manual:
            aws_installer.print_manual_instructions()
            return
        
        # Verificar se já está instalado
        if not force and aws_installer.is_installed():
            version = aws_installer.get_installed_version()
            print(f":white_check_mark: AWS CLI v2 já está instalado (versão {version})")
            
            if not typer.confirm("Deseja reinstalar?"):
                return
        
        # Instalar AWS CLI
        print()
        success = aws_installer.install()
        
        if success:
            print()
            print(":white_check_mark: [bold green]AWS CLI v2 instalado com sucesso![/bold green]")
            print("Teste com: [cyan]aws --version[/cyan]")
        else:
            print()
            print(":x: [bold red]Falha na instalação automática.[/bold red]")
            print()
            aws_installer.print_manual_instructions()
            raise typer.Exit(code=1)
            
    except Exception as e:
        print(f":x: [bold red]Erro inesperado:[/bold red] {e}")
        raise typer.Exit(code=1)


#!/usr/bin/env python3
"""
CLI Leme DevOps - Entry point principal
Ferramenta para configuração automática de ambiente DevOps
"""

import typer
from rich import print
from typing import Optional

from .commands.install_commands import (
    install_docker, uninstall_docker, check_docker_status, 
    system_info, install_terraform, install_azure_cli, install_aws_cli
)
from .commands.environment_commands import setup_environment, environment_status

# --- Configuração da Aplicação ---
app = typer.Typer(
    name="leme",
    help="🚀 CLI Leme DevOps - Configuração automática de ambiente DevOps",
    add_completion=False,
    rich_markup_mode="rich"
)

# --- Callback Principal para Opções Globais ---
@app.callback(invoke_without_command=True)
def main(
    ctx: typer.Context,
    version: Optional[bool] = typer.Option(
        None, 
        "--version", 
        "-v",
        help="Mostra a versão e sai",
        is_eager=True
    ),
    help: Optional[bool] = typer.Option(
        None,
        "--help",
        "-h",
        is_eager=True,
        help="Mostra esta mensagem de ajuda e sai.",
        show_default=False
    )
):
    """
    🚀 CLI Leme DevOps - Automatize a configuração do seu ambiente de desenvolvimento!
    
    Esta ferramenta instala e configura automaticamente todas as ferramentas 
    necessárias para desenvolvimento DevOps, incluindo:
    
    • Docker - Plataforma de containerização
    • Git - Sistema de controle de versão  
    • Terraform - Infraestrutura como código
    • Azure CLI - Interface da Azure
    • AWS CLI - Interface da AWS
    • kubectl - Cliente Kubernetes
    • Ansible - Automação e configuração
    • watch - Execução periódica de comandos
    
    Use 'leme --help' para ver todos os comandos disponíveis.
    """
    if version:
        print("[bold blue]CLI Leme DevOps[/bold blue] [green]v1.0.0[/green]")
        print("Ferramenta para configuração automática de ambiente DevOps")
        raise typer.Exit()
        
    if help:
        typer.echo(ctx.get_help())
        raise typer.Exit()
    
    if ctx.invoked_subcommand is None:
        print("🚀 [bold blue]CLI Leme DevOps[/bold blue] - Bem-vindo!")
        print()
        print("Para começar, use um dos comandos principais:")
        print("  • [cyan]leme setup[/cyan] - Configurar ambiente completo")
        print("  • [cyan]leme status[/cyan] - Ver status das ferramentas")
        print("  • [cyan]leme install --help[/cyan] - Ver opções de instalação")
        print()
        print("Use [cyan]leme --help[/cyan] para ver todos os comandos.")
        raise typer.Exit()


# --- Sub-aplicação para instalação ---
install_app = typer.Typer(
    name="install",
    help="🔧 Instala ferramentas específicas do ambiente DevOps",
    rich_markup_mode="rich"
)
app.add_typer(install_app, name="install")


# --- Comandos de Instalação ---

@install_app.command("docker")
def install_docker_command(
    check_only: bool = typer.Option(False, "--check-only", help="Apenas verificar se o Docker está instalado"),
    force: bool = typer.Option(False, "--force", "-f", help="Forçar reinstalação mesmo se já estiver instalado"),
    manual: bool = typer.Option(False, "--manual", help="Mostrar instruções para instalação manual"),
    no_test: bool = typer.Option(False, "--no-test", help="Não testar a instalação após completar")
):
    """🐳 Instala o Docker automaticamente baseado no sistema operacional."""
    install_docker(check_only, force, manual, no_test)


@install_app.command("azure")
def install_azure_cli_command(
    force: bool = typer.Option(False, "--force", "-f", help="Forçar reinstalação mesmo se já estiver instalado"),
    manual: bool = typer.Option(False, "--manual", help="Mostrar instruções para instalação manual")
):
    """☁️ Instala o Azure CLI automaticamente baseado no sistema operacional."""
    install_azure_cli(force, manual)


@install_app.command("terraform")
def install_terraform_command(
    force: bool = typer.Option(False, "--force", "-f", help="Forçar reinstalação mesmo se já estiver instalado"),
    manual: bool = typer.Option(False, "--manual", help="Mostrar instruções para instalação manual")
):
    """🏗️ Instala o Terraform automaticamente baseado no sistema operacional."""
    install_terraform(force, manual)


@install_app.command("aws")
def install_aws_cli_command(
    force: bool = typer.Option(False, "--force", "-f", help="Forçar reinstalação mesmo se já estiver instalado"),
    manual: bool = typer.Option(False, "--manual", help="Mostrar instruções para instalação manual")
):
    """☁️ Instala o AWS CLI v2 automaticamente baseado no sistema operacional."""
    install_aws_cli(force, manual)


# --- Comandos Principais ---

@app.command("setup")
def setup_command(
    check_only: bool = typer.Option(False, "--check-only", help="Apenas verificar o ambiente atual"),
    required_only: bool = typer.Option(False, "--required-only", help="Instalar apenas ferramentas obrigatórias"),
    skip_docker: bool = typer.Option(False, "--skip-docker", help="Pular instalação do Docker"),
    force: bool = typer.Option(False, "--force", "-f", help="Forçar reinstalação de ferramentas"),
    interactive: bool = typer.Option(False, "--interactive", "-i", help="Modo interativo (LEGACY - agora é padrão)"),
    tools: Optional[str] = typer.Option(None, "--tools", "-t", help="Instalar apenas ferramentas específicas (ex: git,docker)")
):
    """🚀 Configura o ambiente DevOps completo para o curso."""
    tools_list = tools.split(',') if tools else None
    setup_environment(check_only, required_only, skip_docker, force, interactive, tools_list)


@app.command("status")
def status_command():
    """📊 Mostra o status detalhado de todas as ferramentas DevOps."""
    environment_status()


@app.command("info")
def info_command():
    """💻 Mostra informações detalhadas do sistema operacional."""
    system_info()


# --- Comandos de manutenção ---

@app.command("uninstall-docker", hidden=True)
def uninstall_docker_command():
    """🗑️ Remove o Docker do sistema."""
    uninstall_docker()


# --- Aliases para comandos comuns ---
@app.command("check", hidden=True)
def check_command():
    """Alias para 'leme status'"""
    environment_status()


def cli_main():
    """Entry point para o comando leme instalado via pip"""
    app()


if __name__ == "__main__":
    cli_main()


"""Gerenciador principal para instalação do Docker."""

from typing import Optional
from rich import print
import typer

from .system_detector import SystemDetector, SystemInfo, OperatingSystem
from .installers.base_installer import BaseInstaller
from .installers.ubuntu_installer import UbuntuInstaller
from .installers.macos_installer import MacOSInstaller
from .installers.redhat_installer import RedHatInstaller


class DockerInstaller:
    """Gerenciador principal para instalação do Docker."""
    
    def __init__(self):
        """Inicializa o instalador do Docker."""
        self.system_info = SystemDetector.detect()
        self.installer = self._get_installer()
    
    def _get_installer(self) -> Optional[BaseInstaller]:
        """
        Retorna o instalador apropriado para o sistema.
        
        Returns:
            Optional[BaseInstaller]: Instalador ou None se não suportado
        """
        if self.system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            return UbuntuInstaller(self.system_info)
        
        elif self.system_info.os_type == OperatingSystem.MACOS:
            return MacOSInstaller(self.system_info)
        
        elif self.system_info.os_type in [
            OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
        ]:
            return RedHatInstaller(self.system_info)
        
        return None
    
    def install(self, force: bool = False, test_after_install: bool = True) -> bool:
        """
        Instala o Docker no sistema.
        
        Args:
            force: Forçar reinstalação mesmo se já estiver instalado
            test_after_install: Testar instalação após completar
            
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(f":whale: [bold blue]Instalação do Docker[/bold blue]")
        print(f"Sistema detectado: [green]{self.system_info}[/green]")
        print()
        
        # Verificar se o sistema é suportado
        if not self.installer:
            print(f":x: [bold red]Sistema não suportado:[/bold red] {self.system_info.os_type.value}")
            print("Sistemas suportados: Ubuntu, Debian, CentOS, RHEL, Fedora, macOS")
            return False
        
        # Verificar se já está instalado
        if not force and self.installer.is_docker_installed():
            version = self.installer.get_docker_version()
            print(f":white_check_mark: Docker já está instalado (versão {version})")
            
            if not typer.confirm("Deseja reinstalar?"):
                return True
        
        # Verificar pré-requisitos
        if not self.installer.check_prerequisites():
            print(":x: [bold red]Pré-requisitos não atendidos.[/bold red]")
            return False
        
        # Instalar Docker
        print()
        success = self.installer.install()
        
        if not success:
            print()
            print(":x: [bold red]Falha na instalação automática.[/bold red]")
            print()
            self.installer.print_manual_instructions()
            return False
        
        # Testar instalação
        if test_after_install:
            print()
            test_result = self.installer.test_docker_installation()
            if test_result:
                print()
                print(":whale: [bold green]Docker instalado e funcionando![/bold green]")
            else:
                print()
                print(":warning: [bold yellow]Docker instalado mas pode não estar funcionando corretamente.[/bold yellow]")
                self._handle_docker_permission_issues()
        
        return True
    
    def _handle_docker_permission_issues(self) -> None:
        """Lida com problemas comuns de permissão do Docker."""
        print()
        print(":information_source: [bold cyan]Diagnóstico de Problemas Comuns:[/bold cyan]")
        
        # Verificar se é problema de permissão (apenas Linux)
        if self.system_info.os_type != OperatingSystem.MACOS:
            import subprocess
            import os
            
            # Verificar se usuário está no grupo docker
            try:
                result = subprocess.run(["groups"], capture_output=True, text=True)
                groups = result.stdout.strip()
                
                if "docker" not in groups:
                    print(":warning: [yellow]Problema identificado: Usuário não está no grupo 'docker'[/yellow]")
                    print()
                    print("[bold]Soluções:[/bold]")
                    print("1. [blue]Adicionar usuário ao grupo docker:[/blue]")
                    print(f"   sudo usermod -aG docker {os.getenv('USER', 'seu_usuario')}")
                    print("   newgrp docker")
                    print()
                    print("2. [blue]Ou fazer logout/login para aplicar as permissões[/blue]")
                    print()
                    print("3. [blue]Ou reiniciar o sistema[/blue]")
                    print()
                    
                    # Tentar adicionar automaticamente se confirmado
                    import typer
                    if typer.confirm("Deseja tentar adicionar automaticamente ao grupo docker?"):
                        try:
                            subprocess.run([
                                "sudo", "usermod", "-aG", "docker", os.getenv('USER', 'user')
                            ], check=True)
                            print(":white_check_mark: [green]Usuário adicionado ao grupo docker![/green]")
                            print(":information: [blue]Execute 'newgrp docker' ou faça logout/login para aplicar[/blue]")
                        except subprocess.CalledProcessError:
                            print(":x: [red]Falha ao adicionar usuário ao grupo docker[/red]")
                else:
                    print(":white_check_mark: [green]Usuário já está no grupo docker[/green]")
                    
            except Exception:
                pass
        
        # Verificar se Docker daemon está rodando
        try:
            result = subprocess.run(["docker", "version"], capture_output=True, text=True)
            if "Cannot connect to the Docker daemon" in result.stderr:
                print(":warning: [yellow]Docker daemon não está rodando[/yellow]")
                print()
                print("[bold]Soluções:[/bold]")
                if self.system_info.os_type == OperatingSystem.MACOS:
                    print("1. [blue]Abrir Docker Desktop:[/blue]")
                    print("   open /Applications/Docker.app")
                else:
                    print("1. [blue]Iniciar Docker daemon:[/blue]")
                    print("   sudo systemctl start docker")
                    print("   sudo systemctl enable docker")
                print()
        except Exception:
            pass
        
        print("[bold]Para mais ajuda:[/bold]")
        print("• Documentação: https://docs.docker.com/engine/install/linux-postinstall/")
        print("• Tente: python3 main.py environment-status")
    
    def uninstall(self) -> bool:
        """
        Remove o Docker do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        if not self.installer:
            print(f":x: [bold red]Sistema não suportado:[/bold red] {self.system_info.os_type.value}")
            return False
        
        if not self.installer.is_docker_installed():
            print(":information: Docker não está instalado.")
            return True
        
        print(":warning: [bold yellow]Esta ação removerá o Docker completamente do sistema.[/bold yellow]")
        if not typer.confirm("Tem certeza que deseja continuar?"):
            return False
        
        return self.installer.uninstall()
    
    def check_installation(self) -> dict:
        """
        Verifica o status da instalação do Docker.
        
        Returns:
            dict: Informações sobre a instalação
        """
        info = {
            "system": str(self.system_info),
            "supported": self.installer is not None,
            "installed": False,
            "version": None,
            "working": False
        }
        
        if self.installer:
            info["installed"] = self.installer.is_docker_installed()
            if info["installed"]:
                info["version"] = self.installer.get_docker_version()
                info["working"] = self.installer.test_docker_installation()
        
        return info
    
    def print_status(self) -> None:
        """Imprime o status atual da instalação."""
        info = self.check_installation()
        
        print(f":computer: [bold]Sistema:[/bold] {info['system']}")
        print(f":gear: [bold]Suportado:[/bold] {'✓' if info['supported'] else '✗'}")
        
        if info['supported']:
            if info['installed']:
                status_color = "green" if info['working'] else "yellow"
                status_icon = "✓" if info['working'] else "⚠"
                print(f":whale: [bold]Docker:[/bold] [{status_color}]{status_icon} Instalado (v{info['version']})[/{status_color}]")
                
                if not info['working']:
                    print("  [yellow]⚠ Docker pode não estar funcionando corretamente[/yellow]")
            else:
                print(f":whale: [bold]Docker:[/bold] [red]✗ Não instalado[/red]")
        else:
            print("  [red]Sistema não suportado para instalação automática[/red]")
    
    def get_manual_instructions(self) -> None:
        """Imprime instruções para instalação manual."""
        if not self.installer:
            print(f":x: [bold red]Sistema não suportado:[/bold red] {self.system_info.os_type.value}")
            print()
            print("[bold yellow]📋 Instalação Manual:[/bold yellow]")
            print("Visite: https://docs.docker.com/get-docker/")
            return
        
        self.installer.print_manual_instructions()


"""Instalador do Terraform para diferentes sistemas operacionais."""

import subprocess
import os
import tempfile
import zipfile
import stat
from pathlib import Path
from typing import Optional, List
from rich import print

from .base_installer import BaseInstaller
from ..system_detector import SystemInfo, OperatingSystem


class TerraformInstaller(BaseInstaller):
    """Instalador especializado para Terraform."""
    
    def __init__(self, system_info: SystemInfo):
        """
        Inicializa o instalador do Terraform.
        
        Args:
            system_info: Informações do sistema operacional
        """
        super().__init__(system_info)
        self.tool_name = "Terraform"
        
    def is_installed(self) -> bool:
        """
        Verifica se o Terraform está instalado.
        
        Returns:
            bool: True se o Terraform estiver instalado
        """
        try:
            result = subprocess.run(
                ["terraform", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            return result.returncode == 0
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return False
    
    def get_installed_version(self) -> Optional[str]:
        """
        Obtém a versão instalada do Terraform.
        
        Returns:
            Optional[str]: Versão instalada ou None
        """
        try:
            result = subprocess.run(
                ["terraform", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            if result.returncode == 0:
                # Terraform v1.5.7
                first_line = result.stdout.strip().split('\n')[0]
                if "Terraform" in first_line:
                    return first_line.replace("Terraform", "").strip()
            return None
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return None
    
    def install(self) -> bool:
        """
        Instala o Terraform baseado no sistema operacional.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(f":gear: [blue]Instalando {self.tool_name}...[/blue]")
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                return self._install_macos()
            
            elif self.system_info.os_type in [
                OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
                OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
            ]:
                return self._install_ubuntu()
            
            elif self.system_info.os_type in [
                OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
            ]:
                return self._install_redhat()
            
            else:
                print(f":warning: [yellow]Sistema {self.system_info.os_type.value} não suportado para instalação automática do Terraform[/yellow]")
                return self._show_manual_instructions()
        
        except Exception as e:
            print(f":x: [red]Erro durante instalação do Terraform: {str(e)}[/red]")
            return False
    
    def _install_macos(self) -> bool:
        """Instala Terraform no macOS."""
        print(":apple: [blue]Detectado macOS - tentando Homebrew primeiro[/blue]")
        
        # Verificar se Homebrew está disponível
        if self._check_homebrew():
            try:
                print(":beer: [blue]Instalando Terraform via Homebrew...[/blue]")
                result = subprocess.run([
                    "brew", "install", "terraform"
                ], capture_output=True, text=True, timeout=300)
                
                if result.returncode == 0:
                    print(":white_check_mark: [green]Terraform instalado via Homebrew![/green]")
                    return True
                else:
                    print(f":warning: [yellow]Falha no Homebrew: {result.stderr}[/yellow]")
            except subprocess.TimeoutExpired:
                print(":warning: [yellow]Timeout do Homebrew[/yellow]")
        
        # Fallback para download manual
        print(":information: [blue]Tentando instalação via download oficial...[/blue]")
        return self._install_via_download()
    
    def _install_ubuntu(self) -> bool:
        """Instala Terraform no Ubuntu/Debian."""
        print(":gear: [blue]Detectado Ubuntu/Debian - usando repositório HashiCorp[/blue]")
        
        try:
            # Método 1: Tentar repositório oficial HashiCorp
            if self._install_ubuntu_repo():
                return True
            
            # Método 2: Fallback para download manual
            print(":information: [blue]Tentando instalação via download oficial...[/blue]")
            return self._install_via_download()
        
        except Exception as e:
            print(f":warning: [yellow]Erro no método de repositório: {str(e)}[/yellow]")
            return self._install_via_download()
    
    def _install_ubuntu_repo(self) -> bool:
        """Instala via repositório oficial da HashiCorp."""
        try:
            # Instalar dependências
            print(":package: [blue]Instalando dependências...[/blue]")
            subprocess.run([
                "sudo", "apt-get", "update"
            ], check=True, capture_output=True)
            
            subprocess.run([
                "sudo", "apt-get", "install", "-y", "gnupg", "software-properties-common", "curl"
            ], check=True, capture_output=True)
            
            # Adicionar chave GPG da HashiCorp
            print(":key: [blue]Adicionando chave GPG da HashiCorp...[/blue]")
            subprocess.run([
                "sudo", "mkdir", "-p", "/etc/apt/keyrings"
            ], capture_output=True)
            
            subprocess.run([
                "bash", "-c",
                "curl -fsSL https://apt.releases.hashicorp.com/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/hashicorp.gpg"
            ], check=True, capture_output=True)
            
            # Adicionar repositório
            print(":package: [blue]Adicionando repositório HashiCorp...[/blue]")
            
            # Verificar se lsb_release funciona corretamente
            try:
                codename_result = subprocess.run(["lsb_release", "-cs"], capture_output=True, text=True, check=True)
                codename = codename_result.stdout.strip()
                if not codename:
                    raise subprocess.CalledProcessError(1, "lsb_release -cs")
            except subprocess.CalledProcessError:
                # Fallback para distribuições sem lsb_release ou com problemas
                print(":warning: [yellow]lsb_release falhou, usando codename padrão[/yellow]")
                codename = "bookworm"  # Debian 12 padrão
            
            repo_line = f"deb [signed-by=/etc/apt/keyrings/hashicorp.gpg] https://apt.releases.hashicorp.com {codename} main"
            subprocess.run([
                "bash", "-c",
                f"echo '{repo_line}' | sudo tee /etc/apt/sources.list.d/hashicorp.list"
            ], check=True)
            
            # Atualizar e instalar
            print(":arrows_counterclockwise: [blue]Atualizando repositórios...[/blue]")
            subprocess.run(["sudo", "apt-get", "update"], check=True, capture_output=True)
            
            print(":package: [blue]Instalando Terraform...[/blue]")
            subprocess.run(["sudo", "apt-get", "install", "-y", "terraform"], check=True)
            
            print(":white_check_mark: [green]Terraform instalado via repositório HashiCorp![/green]")
            return True
        
        except subprocess.CalledProcessError as e:
            print(f":warning: [yellow]Falha no repositório HashiCorp: {e}[/yellow]")
            # Limpar repositório corrompido para não afetar outras instalações
            self._cleanup_failed_repository()
            return False
    
    def _install_redhat(self) -> bool:
        """Instala Terraform no CentOS/RHEL/Fedora."""
        print(":gear: [blue]Detectado sistema RedHat - usando repositório HashiCorp[/blue]")
        
        try:
            # Determinar gerenciador de pacotes
            if self.system_info.os_type == OperatingSystem.FEDORA:
                pkg_manager = "dnf"
            else:
                pkg_manager = "yum"
            
            # Instalar yum-utils se necessário
            subprocess.run([
                "sudo", pkg_manager, "install", "-y", "yum-utils"
            ], check=True, capture_output=True)
            
            # Adicionar repositório HashiCorp
            print(":package: [blue]Adicionando repositório HashiCorp...[/blue]")
            subprocess.run([
                "sudo", "yum-config-manager", "--add-repo", "https://rpm.releases.hashicorp.com/RHEL/hashicorp.repo"
            ], check=True)
            
            # Instalar Terraform
            print(":package: [blue]Instalando Terraform...[/blue]")
            subprocess.run([
                "sudo", pkg_manager, "install", "-y", "terraform"
            ], check=True)
            
            print(":white_check_mark: [green]Terraform instalado via repositório HashiCorp![/green]")
            return True
        
        except subprocess.CalledProcessError as e:
            print(f":warning: [yellow]Falha no repositório: {e}[/yellow]")
            return self._install_via_download()
    
    def _install_via_download(self) -> bool:
        """Instala Terraform via download direto (método universal)."""
        print(":globe_with_meridians: [blue]Instalando via download oficial da HashiCorp...[/blue]")
        
        try:
            # Detectar arquitetura e SO
            arch = self._get_architecture()
            os_name = self._get_os_name()
            
            if not arch or not os_name:
                print(":x: [red]Não foi possível detectar arquitetura ou SO[/red]")
                return False
            
            # URL de download (sempre pegar a versão mais recente seria ideal, mas vamos usar uma estável)
            version = "1.5.7"  # Versão estável conhecida
            url = f"https://releases.hashicorp.com/terraform/{version}/terraform_{version}_{os_name}_{arch}.zip"
            
            with tempfile.TemporaryDirectory() as temp_dir:
                temp_path = Path(temp_dir)
                zip_file = temp_path / f"terraform_{version}.zip"
                
                # Download
                print(f":arrow_down: [blue]Baixando Terraform {version} para {os_name} {arch}...[/blue]")
                result = subprocess.run([
                    "curl", "-L", "-o", str(zip_file), url
                ], capture_output=True)
                
                if result.returncode != 0:
                    print(":x: [red]Falha no download[/red]")
                    return False
                
                # Extrair
                print(":package: [blue]Extraindo arquivo...[/blue]")
                with zipfile.ZipFile(zip_file, 'r') as zip_ref:
                    zip_ref.extractall(temp_path)
                
                # Instalar
                terraform_binary = temp_path / "terraform"
                if not terraform_binary.exists():
                    print(":x: [red]Binário do Terraform não encontrado no arquivo[/red]")
                    return False
                
                # Copiar para /usr/local/bin
                install_path = Path("/usr/local/bin/terraform")
                print(f":gear: [blue]Instalando em {install_path}...[/blue]")
                
                subprocess.run([
                    "sudo", "cp", str(terraform_binary), str(install_path)
                ], check=True)
                
                subprocess.run([
                    "sudo", "chmod", "+x", str(install_path)
                ], check=True)
                
                print(":white_check_mark: [green]Terraform instalado via download oficial![/green]")
                return True
        
        except Exception as e:
            print(f":x: [red]Erro na instalação via download: {str(e)}[/red]")
            return False
    
    def _get_architecture(self) -> Optional[str]:
        """Retorna a arquitetura para download."""
        try:
            result = subprocess.run(["uname", "-m"], capture_output=True, text=True)
            if result.returncode == 0:
                arch = result.stdout.strip()
                if arch in ["x86_64", "amd64"]:
                    return "amd64"
                elif arch in ["aarch64", "arm64"]:
                    return "arm64"
                elif arch in ["arm"]:
                    return "arm"
        except Exception:
            pass
        return None
    
    def _get_os_name(self) -> Optional[str]:
        """Retorna o nome do SO para download."""
        if self.system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN,
            OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
        ]:
            return "linux"
        elif self.system_info.os_type == OperatingSystem.MACOS:
            return "darwin"
        return None
    
    def _check_homebrew(self) -> bool:
        """Verifica se o Homebrew está instalado."""
        try:
            subprocess.run(["brew", "--version"], capture_output=True, check=True)
            return True
        except (FileNotFoundError, subprocess.CalledProcessError):
            return False
    
    def _show_manual_instructions(self) -> bool:
        """Mostra instruções para instalação manual."""
        print(":information_source: [cyan]Instruções para instalação manual do Terraform:[/cyan]")
        print()
        print("1. Acesse: https://www.terraform.io/downloads")
        print("2. Baixe o arquivo ZIP para seu sistema operacional")
        print("3. Extraia o arquivo e mova o binário para um diretório no PATH")
        print("4. Verifique a instalação com: terraform --version")
        print()
        return False
    
    def get_install_commands(self) -> List[str]:
        """
        Retorna lista de comandos para instalação manual.
        
        Returns:
            List[str]: Lista de comandos
        """
        if self.system_info.os_type == OperatingSystem.MACOS:
            return [
                "# Via Homebrew (recomendado)",
                "brew install terraform",
                "",
                "# Ou via download manual:",
                "# 1. Acesse https://www.terraform.io/downloads",
                "# 2. Baixe o ZIP para macOS",
                "# 3. Extraia e mova para /usr/local/bin"
            ]
        
        elif self.system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            return [
                "# Via repositório HashiCorp",
                "sudo apt-get update && sudo apt-get install -y gnupg software-properties-common curl",
                "curl -fsSL https://apt.releases.hashicorp.com/gpg | sudo gpg --dearmor -o /etc/apt/keyrings/hashicorp.gpg",
                "echo 'deb [signed-by=/etc/apt/keyrings/hashicorp.gpg] https://apt.releases.hashicorp.com $(lsb_release -cs) main' | sudo tee /etc/apt/sources.list.d/hashicorp.list",
                "sudo apt-get update && sudo apt-get install terraform"
            ]
        
        elif self.system_info.os_type in [
            OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
        ]:
            pkg_manager = "dnf" if self.system_info.os_type == OperatingSystem.FEDORA else "yum"
            return [
                "# Via repositório HashiCorp",
                f"sudo {pkg_manager} install -y yum-utils",
                "sudo yum-config-manager --add-repo https://rpm.releases.hashicorp.com/RHEL/hashicorp.repo",
                f"sudo {pkg_manager} install terraform"
            ]
        
        else:
            return [
                "# Instalação manual",
                "1. Acesse: https://www.terraform.io/downloads",
                "2. Baixe o arquivo ZIP para seu sistema operacional",
                "3. Extraia o arquivo e mova o binário para um diretório no PATH",
                "4. Verifique com: terraform --version"
            ]

    def uninstall(self) -> bool:
        """
        Remove o Terraform do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        print(f":wastebasket: [blue]Removendo {self.tool_name}...[/blue]")
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                # Tentar remover via Homebrew primeiro
                try:
                    subprocess.run(["brew", "uninstall", "terraform"], check=True)
                    print(":white_check_mark: [green]Terraform removido via Homebrew![/green]")
                    return True
                except subprocess.CalledProcessError:
                    pass
            
            elif self.system_info.os_type in [
                OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
                OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
            ]:
                try:
                    subprocess.run(["sudo", "apt-get", "remove", "-y", "terraform"], check=True)
                    print(":white_check_mark: [green]Terraform removido via apt![/green]")
                    return True
                except subprocess.CalledProcessError:
                    pass
            
            elif self.system_info.os_type in [
                OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
            ]:
                pkg_manager = "dnf" if self.system_info.os_type == OperatingSystem.FEDORA else "yum"
                try:
                    subprocess.run(["sudo", pkg_manager, "remove", "-y", "terraform"], check=True)
                    print(f":white_check_mark: [green]Terraform removido via {pkg_manager}![/green]")
                    return True
                except subprocess.CalledProcessError:
                    pass
            
            # Tentar remover binário manual
            binary_path = Path("/usr/local/bin/terraform")
            if binary_path.exists():
                subprocess.run(["sudo", "rm", str(binary_path)], check=True)
                print(":white_check_mark: [green]Terraform removido (binário manual)![/green]")
                return True
            
            print(":x: [red]Terraform não encontrado para remoção[/red]")
            return False
        
        except Exception as e:
            print(f":x: [red]Erro ao remover Terraform: {str(e)}[/red]")
            return False
    
    def _cleanup_failed_repository(self) -> None:
        """Remove repositório HashiCorp corrompido para não afetar outras instalações."""
        try:
            print(":broom: [blue]Limpando repositório corrompido...[/blue]")
            
            # Remover arquivo de repositório se existir
            subprocess.run([
                "sudo", "rm", "-f", "/etc/apt/sources.list.d/hashicorp.list"
            ], capture_output=True)
            
            # Remover chave GPG se existir
            subprocess.run([
                "sudo", "rm", "-f", "/etc/apt/keyrings/hashicorp.gpg"
            ], capture_output=True)
            
            # Tentar atualizar repositórios para limpar cache
            subprocess.run([
                "sudo", "apt-get", "update"
            ], capture_output=True)
            
            print(":white_check_mark: [green]Repositório limpo com sucesso[/green]")
            
        except Exception as e:
            print(f":warning: [yellow]Aviso: Não foi possível limpar repositório: {str(e)}[/yellow]")


"""Instalador do Git para diferentes sistemas operacionais."""

import subprocess
from typing import Optional, List
from rich import print

from .base_installer import BaseInstaller
from ..system_detector import SystemInfo, OperatingSystem


class GitInstaller(BaseInstaller):
    """Instalador especializado para Git."""
    
    def __init__(self, system_info: SystemInfo):
        """
        Inicializa o instalador do Git.
        
        Args:
            system_info: Informações do sistema operacional
        """
        super().__init__(system_info)
        self.tool_name = "Git"
        
    def is_installed(self) -> bool:
        """
        Verifica se o Git está instalado.
        
        Returns:
            bool: True se o Git estiver instalado
        """
        try:
            result = subprocess.run(
                ["git", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            return result.returncode == 0
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return False
    
    def get_installed_version(self) -> Optional[str]:
        """
        Obtém a versão instalada do Git.
        
        Returns:
            Optional[str]: Versão instalada ou None
        """
        try:
            result = subprocess.run(
                ["git", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            if result.returncode == 0:
                # git version 2.39.2
                output = result.stdout.strip()
                if "git version" in output:
                    return output.replace("git version", "").strip()
            return None
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return None
    
    def install(self) -> bool:
        """
        Instala o Git baseado no sistema operacional.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(f":gear: [blue]Instalando {self.tool_name}...[/blue]")
        
        # Verificar se já está instalado
        if self.is_installed():
            version = self.get_installed_version()
            print(f":white_check_mark: [green]Git já está instalado (versão {version})[/green]")
            return True
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                return self._install_macos()
            
            elif self.system_info.os_type in [
                OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
                OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
            ]:
                return self._install_ubuntu()
            
            elif self.system_info.os_type in [
                OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
            ]:
                return self._install_redhat()
            
            else:
                print(f":warning: [yellow]Sistema {self.system_info.os_type.value} não suportado para instalação automática do Git[/yellow]")
                return self._show_manual_instructions()
        
        except Exception as e:
            print(f":x: [red]Erro durante instalação do Git: {str(e)}[/red]")
            return False
    
    def _install_macos(self) -> bool:
        """Instala Git no macOS."""
        print(":apple: [blue]Detectado macOS[/blue]")
        
        # Verificar se Xcode Command Line Tools estão instalados
        if self._check_xcode_tools():
            print(":white_check_mark: [green]Git já disponível via Xcode Command Line Tools![/green]")
            return True
        
        # Verificar se Homebrew está disponível
        if self._check_homebrew():
            return self._install_via_homebrew()
        
        # Instalar Xcode Command Line Tools
        return self._install_xcode_tools()
    
    def _install_ubuntu(self) -> bool:
        """Instala Git no Ubuntu/Debian."""
        print(":gear: [blue]Detectado Ubuntu/Debian - usando apt[/blue]")
        
        try:
            # Atualizar repositórios
            print(":arrows_counterclockwise: [blue]Atualizando repositórios...[/blue]")
            subprocess.run(["sudo", "apt", "update"], check=True, capture_output=True)
            
            # Instalar Git
            print(":package: [blue]Instalando Git...[/blue]")
            subprocess.run(["sudo", "apt", "install", "-y", "git"], check=True)
            
            # Verificar instalação
            if self.is_installed():
                version = self.get_installed_version()
                print(f":white_check_mark: [green]Git {version} instalado com sucesso![/green]")
                return True
            else:
                print(":x: [red]Falha na verificação pós-instalação[/red]")
                return False
        
        except subprocess.CalledProcessError as e:
            print(f":x: [red]Erro na instalação via apt: {e}[/red]")
            return self._show_manual_instructions()
    
    def _install_redhat(self) -> bool:
        """Instala Git no CentOS/RHEL/Fedora."""
        print(":gear: [blue]Detectado sistema RedHat[/blue]")
        
        try:
            # Determinar gerenciador de pacotes
            if self.system_info.os_type == OperatingSystem.FEDORA:
                pkg_manager = "dnf"
            else:
                pkg_manager = "yum"
            
            print(f":package: [blue]Instalando Git via {pkg_manager}...[/blue]")
            subprocess.run(["sudo", pkg_manager, "install", "-y", "git"], check=True)
            
            # Verificar instalação
            if self.is_installed():
                version = self.get_installed_version()
                print(f":white_check_mark: [green]Git {version} instalado com sucesso![/green]")
                return True
            else:
                print(":x: [red]Falha na verificação pós-instalação[/red]")
                return False
        
        except subprocess.CalledProcessError as e:
            print(f":x: [red]Erro na instalação via {pkg_manager}: {e}[/red]")
            return self._show_manual_instructions()
    
    def _install_via_homebrew(self) -> bool:
        """Instala Git via Homebrew no macOS."""
        print(":beer: [blue]Instalando Git via Homebrew...[/blue]")
        
        try:
            subprocess.run(["brew", "install", "git"], check=True, timeout=300)
            
            if self.is_installed():
                version = self.get_installed_version()
                print(f":white_check_mark: [green]Git {version} instalado via Homebrew![/green]")
                return True
            else:
                print(":x: [red]Falha na verificação pós-instalação[/red]")
                return False
        
        except subprocess.CalledProcessError as e:
            print(f":warning: [yellow]Falha no Homebrew: {e}[/yellow]")
            return self._install_xcode_tools()
        except subprocess.TimeoutExpired:
            print(":warning: [yellow]Timeout do Homebrew[/yellow]")
            return self._install_xcode_tools()
    
    def _install_xcode_tools(self) -> bool:
        """Instala Xcode Command Line Tools (inclui Git)."""
        print(":hammer_and_wrench: [blue]Instalando Xcode Command Line Tools...[/blue]")
        
        try:
            # Tentar instalar Xcode Command Line Tools
            result = subprocess.run([
                "xcode-select", "--install"
            ], capture_output=True, text=True)
            
            if result.returncode == 0:
                print(":information: [blue]Instalação do Xcode Command Line Tools iniciada[/blue]")
                print(":information: [yellow]Uma janela será aberta - siga as instruções para completar a instalação[/yellow]")
                print(":information: [yellow]Após a instalação, execute novamente este comando[/yellow]")
                return False  # Instalação requer interação do usuário
            
            elif "already installed" in result.stderr:
                print(":white_check_mark: [green]Xcode Command Line Tools já instalados![/green]")
                return self.is_installed()
            
            else:
                print(f":x: [red]Falha ao instalar Xcode Command Line Tools: {result.stderr}[/red]")
                return self._show_manual_instructions()
        
        except Exception as e:
            print(f":x: [red]Erro ao instalar Xcode Command Line Tools: {str(e)}[/red]")
            return self._show_manual_instructions()
    
    def _check_xcode_tools(self) -> bool:
        """Verifica se Xcode Command Line Tools estão instalados."""
        try:
            result = subprocess.run([
                "xcode-select", "-p"
            ], capture_output=True, text=True)
            return result.returncode == 0
        except:
            return False
    
    def _check_homebrew(self) -> bool:
        """Verifica se o Homebrew está instalado."""
        try:
            subprocess.run(["brew", "--version"], capture_output=True, check=True)
            return True
        except (FileNotFoundError, subprocess.CalledProcessError):
            return False
    
    def _show_manual_instructions(self) -> bool:
        """Mostra instruções para instalação manual."""
        print(":information_source: [cyan]Instruções para instalação manual do Git:[/cyan]")
        print()
        
        if self.system_info.os_type == OperatingSystem.MACOS:
            print("macOS:")
            print("1. Instalar Xcode Command Line Tools:")
            print("   xcode-select --install")
            print("2. Ou baixar o Git em: https://git-scm.com/download/mac")
            print("3. Ou instalar Homebrew e executar: brew install git")
        
        elif self.system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            print("Ubuntu/Debian:")
            print("1. sudo apt update")
            print("2. sudo apt install git")
        
        elif self.system_info.os_type in [
            OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
        ]:
            if self.system_info.os_type == OperatingSystem.FEDORA:
                print("Fedora:")
                print("1. sudo dnf install git")
            else:
                print("CentOS/RHEL:")
                print("1. sudo yum install git")
        
        else:
            print("Geral:")
            print("1. Acesse: https://git-scm.com/downloads")
            print("2. Baixe a versão para seu sistema operacional")
            print("3. Siga as instruções de instalação")
        
        print()
        return False
    
    def get_install_commands(self) -> List[str]:
        """
        Retorna lista de comandos para instalação manual.
        
        Returns:
            List[str]: Lista de comandos
        """
        if self.system_info.os_type == OperatingSystem.MACOS:
            return [
                "# Via Xcode Command Line Tools (recomendado)",
                "xcode-select --install",
                "",
                "# Ou via Homebrew:",
                "brew install git",
                "",
                "# Ou baixar instalador:",
                "# https://git-scm.com/download/mac"
            ]
        
        elif self.system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            return [
                "sudo apt update",
                "sudo apt install git"
            ]
        
        elif self.system_info.os_type in [
            OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
        ]:
            if self.system_info.os_type == OperatingSystem.FEDORA:
                return ["sudo dnf install git"]
            else:
                return ["sudo yum install git"]
        
        else:
            return [
                "# Acesse: https://git-scm.com/downloads",
                "# Baixe e instale a versão para seu sistema operacional"
            ]

    def uninstall(self) -> bool:
        """
        Remove o Git do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        print(f":wastebasket: [blue]Removendo {self.tool_name}...[/blue]")
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                # No macOS, Git geralmente vem com Xcode Tools ou Homebrew
                if self._check_homebrew():
                    try:
                        subprocess.run(["brew", "uninstall", "git"], check=True)
                        print(":white_check_mark: [green]Git removido via Homebrew![/green]")
                        return True
                    except subprocess.CalledProcessError:
                        pass
                
                print(":information: [yellow]No macOS, Git geralmente vem com Xcode Command Line Tools[/yellow]")
                print(":information: [yellow]Para remover completamente, desinstale Xcode Command Line Tools[/yellow]")
                return False
            
            elif self.system_info.os_type in [
                OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
                OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
            ]:
                subprocess.run(["sudo", "apt", "remove", "-y", "git"], check=True)
                print(":white_check_mark: [green]Git removido via apt![/green]")
                return True
            
            elif self.system_info.os_type in [
                OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
            ]:
                pkg_manager = "dnf" if self.system_info.os_type == OperatingSystem.FEDORA else "yum"
                subprocess.run(["sudo", pkg_manager, "remove", "-y", "git"], check=True)
                print(f":white_check_mark: [green]Git removido via {pkg_manager}![/green]")
                return True
            
            else:
                print(":information: [yellow]Remoção manual necessária para este sistema[/yellow]")
                return False
        
        except subprocess.CalledProcessError as e:
            print(f":x: [red]Erro ao remover Git: {e}[/red]")
            return False
        except Exception as e:
            print(f":x: [red]Erro ao remover Git: {str(e)}[/red]")
            return False
    
    def configure_git(self, name: Optional[str] = None, email: Optional[str] = None) -> bool:
        """
        Configura Git com nome e email do usuário.
        
        Args:
            name: Nome do usuário
            email: Email do usuário
            
        Returns:
            bool: True se a configuração foi bem-sucedida
        """
        if not self.is_installed():
            print(":x: [red]Git não está instalado[/red]")
            return False
        
        try:
            print(":gear: [blue]Configurando Git...[/blue]")
            
            if name:
                subprocess.run(["git", "config", "--global", "user.name", name], check=True)
                print(f":white_check_mark: [green]Nome configurado: {name}[/green]")
            
            if email:
                subprocess.run(["git", "config", "--global", "user.email", email], check=True)
                print(f":white_check_mark: [green]Email configurado: {email}[/green]")
            
            # Configurações recomendadas
            subprocess.run(["git", "config", "--global", "init.defaultBranch", "main"], check=True)
            subprocess.run(["git", "config", "--global", "pull.rebase", "false"], check=True)
            
            print(":white_check_mark: [green]Git configurado com sucesso![/green]")
            return True
        
        except subprocess.CalledProcessError as e:
            print(f":x: [red]Erro ao configurar Git: {e}[/red]")
            return False
    
    def show_config(self) -> bool:
        """
        Mostra a configuração atual do Git.
        
        Returns:
            bool: True se conseguiu mostrar a configuração
        """
        if not self.is_installed():
            print(":x: [red]Git não está instalado[/red]")
            return False
        
        try:
            print(":gear: [blue]Configuração atual do Git:[/blue]")
            
            # Nome
            try:
                result = subprocess.run(["git", "config", "--global", "user.name"], 
                                     capture_output=True, text=True, check=True)
                print(f":person: [green]Nome: {result.stdout.strip()}[/green]")
            except subprocess.CalledProcessError:
                print(":warning: [yellow]Nome não configurado[/yellow]")
            
            # Email
            try:
                result = subprocess.run(["git", "config", "--global", "user.email"], 
                                     capture_output=True, text=True, check=True)
                print(f":email: [green]Email: {result.stdout.strip()}[/green]")
            except subprocess.CalledProcessError:
                print(":warning: [yellow]Email não configurado[/yellow]")
            
            return True
        
        except Exception as e:
            print(f":x: [red]Erro ao mostrar configuração: {str(e)}[/red]")
            return False


"""Instalador do Docker para macOS."""

import subprocess
import shutil
from typing import List
from rich import print

from .base_installer import BaseInstaller


class MacOSInstaller(BaseInstaller):
    """Instalador do Docker para macOS."""
    
    def install(self) -> bool:
        """
        Instala o Docker no macOS usando Homebrew.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(":apple: Instalando Docker no macOS...")
        
        try:
            # Verificar se Homebrew está instalado
            if not self._check_homebrew():
                return False
            
            # 1. Atualizar Homebrew
            print("  [blue]1/3[/blue] Atualizando Homebrew...")
            self._run_command(["brew", "update"])
            
            # 2. Instalar Docker
            print("  [blue]2/3[/blue] Instalando Docker...")
            self._run_command(["brew", "install", "--cask", "docker"])
            
            # 3. Instruções para iniciar Docker
            print("  [blue]3/3[/blue] Configuração finalizada!")
            print()
            print("[yellow]📝 Próximos passos:[/yellow]")
            print("  1. Abra o Docker Desktop na pasta Applications")
            print("  2. Aguarde o Docker inicializar (pode demorar alguns minutos)")
            print("  3. O Docker estará pronto para uso!")
            print()
            
            return True
            
        except subprocess.CalledProcessError as e:
            print(f"  [red]✗[/red] Erro durante a instalação: {e}")
            return False
        except Exception as e:
            print(f"  [red]✗[/red] Erro inesperado: {e}")
            return False
    
    def _check_homebrew(self) -> bool:
        """Verifica se o Homebrew está instalado."""
        if shutil.which("brew"):
            return True
        
        print("  [yellow]![/yellow] Homebrew não encontrado.")
        print("  [blue]📦[/blue] Instalando Homebrew...")
        
        try:
            # Instalar Homebrew
            install_cmd = [
                "/bin/bash", "-c", 
                "\"$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)\""
            ]
            self._run_command(install_cmd, shell=True)
            
            print("  [green]✓[/green] Homebrew instalado com sucesso!")
            return True
            
        except Exception as e:
            print(f"  [red]✗[/red] Erro ao instalar Homebrew: {e}")
            print("  [yellow]💡[/yellow] Instale manualmente: https://brew.sh")
            return False
    
    def uninstall(self) -> bool:
        """
        Remove o Docker do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        try:
            print(":wastebasket: Removendo Docker...")
            
            # Remover Docker Desktop
            self._run_command(["brew", "uninstall", "--cask", "docker"], ignore_errors=True)
            
            print("  [green]✓[/green] Docker removido com sucesso!")
            return True
            
        except Exception as e:
            print(f"  [red]✗[/red] Erro durante a remoção: {e}")
            return False
    
    def get_install_commands(self) -> List[str]:
        """Retorna lista de comandos para instalação manual."""
        return [
            "# Instalar Homebrew (se não estiver instalado)",
            '/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"',
            "",
            "# Instalar Docker",
            "brew update",
            "brew install --cask docker",
            "",
            "# Abrir Docker Desktop",
            "open /Applications/Docker.app"
        ]
    
    def test_docker_installation(self) -> bool:
        """
        Testa se o Docker está funcionando no macOS.
        
        Returns:
            bool: True se o Docker está funcionando
        """
        # No macOS, o Docker Desktop precisa estar rodando
        if not self.is_docker_installed():
            return False
        
        print("  [blue]🧪[/blue] Verificando se Docker Desktop está rodando...")
        
        try:
            # Verificar se Docker daemon está ativo
            result = subprocess.run(
                ["docker", "info"],
                capture_output=True,
                text=True,
                timeout=10
            )
            
            if result.returncode != 0:
                print("  [yellow]![/yellow] Docker Desktop não está rodando")
                print("  [blue]💡[/blue] Inicie o Docker Desktop manualmente")
                return False
            
            return super().test_docker_installation()
            
        except Exception as e:
            print(f"  [yellow]![/yellow] Erro ao verificar Docker: {e}")
            return False


"""Comandos para configuração do ambiente DevOps."""

import subprocess
import typer
from rich import print
from typing import Optional, List

from ..system.environment_manager import EnvironmentManager
from ..system.docker_installer import DockerInstaller
from ..system.installers.git_installer import GitInstaller
from ..system.installers.terraform_installer import TerraformInstaller
from ..system.installers.aws_cli_installer import AwsCliInstaller
from ..system.installers.azure_cli_installer import AzureCliInstaller
from ..config.constants import Tool, DEVOPS_TOOLS_CONFIG


def setup_environment(
    check_only: bool = typer.Option(False, "--check-only", help="Apenas verificar o ambiente atual"),
    required_only: bool = typer.Option(False, "--required-only", help="Instalar apenas ferramentas obrigatórias"),
    skip_docker: bool = typer.Option(False, "--skip-docker", help="Pular instalação do Docker"),
    force: bool = typer.Option(False, "--force", "-f", help="Forçar reinstalação de ferramentas"),
    interactive: bool = typer.Option(False, "--interactive", "-i", help="Modo interativo (LEGACY - agora é padrão)"),
    tools: Optional[List[str]] = typer.Option(None, "--tools", "-t", help="Instalar apenas ferramentas específicas (ex: git,docker)")
) -> None:
    """
    Configura o ambiente DevOps completo para o curso.
    
    Este comando verifica e instala todas as ferramentas necessárias:
    - Docker (obrigatório)
    - Git (obrigatório) 
    - Azure CLI (opcional)
    - AWS CLI v2 (opcional)
    - kubectl (opcional)
    - Ansible (opcional)
    - watch (opcional)
    """
    print(":rocket: [bold green]Setup do Ambiente DevOps[/bold green]")
    print()
    
    # Inicializar gerenciador
    env_manager = EnvironmentManager()
    
    # Modo apenas verificação
    if check_only:
        env_manager.show_status_report()
        return
    
    # Verificar status atual
    print(":mag: [bold blue]Verificando ambiente atual...[/bold blue]")
    env_manager.check_all_tools()
    env_manager.show_status_report()
    
    # Determinar ferramentas a instalar
    if tools:
        # Lista específica de ferramentas
        selected_tools = []
        for tool_name in tools:
            try:
                tool = Tool(tool_name.lower())
                selected_tools.append(tool)
            except ValueError:
                print(f":warning: [yellow]Ferramenta desconhecida: {tool_name}[/yellow]")
        
        if not selected_tools:
            print(":x: [red]Nenhuma ferramenta válida especificada[/red]")
            raise typer.Exit(1)
        
        tools_to_install = [t for t in selected_tools if not env_manager.tools_status.get(t, None) or not env_manager.tools_status[t].installed]
    
    elif required_only:
        # Apenas ferramentas obrigatórias
        tools_to_install = env_manager.get_missing_tools(only_required=True)
    
    else:
        # Todas as ferramentas não instaladas
        tools_to_install = env_manager.get_missing_tools(only_required=False)
    
    # Filtrar Docker se solicitado
    if skip_docker and Tool.DOCKER in tools_to_install:
        tools_to_install.remove(Tool.DOCKER)
        print(":information: [blue]Docker será pulado conforme solicitado[/blue]")
    
    # Verificar se há algo para instalar
    if not tools_to_install:
        print(":white_check_mark: [green]Todas as ferramentas selecionadas já estão instaladas![/green]")
        return
    
    # Sempre perguntar para ferramentas opcionais (exceto se --force)
    if not force:
        print(f"\n:question: [bold cyan]Escolha as ferramentas para instalar:[/bold cyan]")
        selected_tools = []
        
        for tool in tools_to_install:
            config = DEVOPS_TOOLS_CONFIG[tool]
            required_text = "[red](obrigatória)[/red]" if config["required"] else "[yellow](opcional)[/yellow]"
            
            print(f"\n• [blue]{config['name']}[/blue] {required_text}")
            print(f"  {config['description']}")
            
            # Agora todas as ferramentas são opcionais - perguntar para todas
            confirm = typer.confirm(f"  Deseja instalar {config['name']}?")
            if confirm:
                selected_tools.append(tool)
            else:
                print(f"  :information: [yellow]Pulando {config['name']}[/yellow]")
        
        tools_to_install = selected_tools
        
        if not tools_to_install:
            print("\n:information: [yellow]Nenhuma ferramenta selecionada para instalação[/yellow]")
            return
        
        print(f"\n:white_check_mark: [bold green]Ferramentas selecionadas para instalação:[/bold green]")
        for tool in tools_to_install:
            config = DEVOPS_TOOLS_CONFIG[tool]
            print(f"  • [blue]{config['name']}[/blue]")
    
    else:
        # Modo --force - instalar tudo sem perguntar
        print(f"\n:wrench: [bold cyan]Modo Force - Instalando todas as ferramentas:[/bold cyan]")
        for tool in tools_to_install:
            config = DEVOPS_TOOLS_CONFIG[tool]
            required_text = "[red](obrigatória)[/red]" if config["required"] else "[yellow](opcional)[/yellow]"
            print(f"  • [blue]{config['name']}[/blue] {required_text} - {config['description']}")
    
    print("\n:gear: [bold green]Iniciando instalação das ferramentas...[/bold green]")
    
    # Instalar ferramentas uma por vez
    success_count = 0
    for tool in tools_to_install:
        config = DEVOPS_TOOLS_CONFIG[tool]
        print(f"\n:arrow_forward: [bold blue]Instalando {config['name']}...[/bold blue]")
        
        try:
            success = _install_tool(tool, env_manager.system_info, force)
            if success:
                print(f":white_check_mark: [green]{config['name']} instalado com sucesso![/green]")
                success_count += 1
            else:
                print(f":x: [red]Falha ao instalar {config['name']}[/red]")
        
        except Exception as e:
            print(f":x: [red]Erro ao instalar {config['name']}: {str(e)}[/red]")
    
    # Relatório final
    print(f"\n:chart_with_upwards_trend: [bold cyan]Relatório de Instalação:[/bold cyan]")
    print(f"  • [green]Instaladas com sucesso:[/green] {success_count}")
    print(f"  • [red]Falharam:[/red] {len(tools_to_install) - success_count}")
    
    # Verificar ambiente final
    print("\n:mag: [bold blue]Verificando ambiente após instalação...[/bold blue]")
    env_manager.check_all_tools()
    env_manager.show_status_report()
    
    # Status final
    if env_manager.is_environment_ready():
        print("\n:party_popper: [bold green]Ambiente DevOps configurado com sucesso![/bold green]")
        print(":information: [blue]Você está pronto para o curso![/blue]")
    else:
        missing = env_manager.get_missing_tools(only_required=True)
        print(f"\n:warning: [yellow]Ainda faltam algumas ferramentas obrigatórias: {[DEVOPS_TOOLS_CONFIG[t]['name'] for t in missing]}[/yellow]")
        print(":information: [blue]Execute novamente o comando para tentar instalar as ferramentas em falta[/blue]")


def environment_status() -> None:
    """Mostra o status atual de todas as ferramentas DevOps."""
    env_manager = EnvironmentManager()
    env_manager.show_status_report()


def _install_tool(tool: Tool, system_info, force: bool = False) -> bool:
    """
    Instala uma ferramenta específica.
    
    Args:
        tool: Ferramenta a ser instalada
        system_info: Informações do sistema
        force: Forçar reinstalação
        
    Returns:
        bool: True se a instalação foi bem-sucedida
    """
    try:
        if tool == Tool.DOCKER:
            # Docker já tem instalador dedicado
            docker_installer = DockerInstaller()
            return docker_installer.install(force=force, test_after_install=True)
        
        elif tool == Tool.GIT:
            return _install_git(system_info)
        
        elif tool == Tool.TERRAFORM:
            return _install_terraform(system_info)
        
        elif tool == Tool.AZURE_CLI:
            return _install_azure_cli(system_info)
        
        elif tool == Tool.AWS_CLI:
            return _install_aws_cli(system_info)
        
        elif tool == Tool.KUBECTL:
            return _install_kubectl(system_info)
        
        elif tool == Tool.ANSIBLE:
            return _install_ansible(system_info)
        
        elif tool == Tool.WATCH:
            return _install_watch(system_info)
        
        else:
            print(f":warning: [yellow]Instalador para {tool.value} não implementado ainda[/yellow]")
            return False
    
    except Exception as e:
        print(f":x: [red]Erro durante instalação de {tool.value}: {str(e)}[/red]")
        return False


def _install_git(system_info) -> bool:
    """Instala Git baseado no sistema operacional."""
    try:
        git_installer = GitInstaller(system_info)
        return git_installer.install()
    except Exception as e:
        print(f":x: [red]Erro durante instalação do Git: {str(e)}[/red]")
        return False


def _install_terraform(system_info) -> bool:
    """Instala Terraform baseado no sistema operacional."""
    try:
        terraform_installer = TerraformInstaller(system_info)
        return terraform_installer.install()
    except Exception as e:
        print(f":x: [red]Erro durante instalação do Terraform: {str(e)}[/red]")
        return False


def _install_azure_cli(system_info) -> bool:
    """Instala Azure CLI baseado no sistema operacional."""
    try:
        azure_installer = AzureCliInstaller(system_info)
        return azure_installer.install()
    except Exception as e:
        print(f":x: [red]Erro durante instalação do Azure CLI: {str(e)}[/red]")
        return False


def _install_aws_cli(system_info) -> bool:
    """Instala AWS CLI v2 baseado no sistema operacional."""
    try:
        aws_installer = AwsCliInstaller(system_info)
        return aws_installer.install()
    except Exception as e:
        print(f":x: [red]Erro durante instalação do AWS CLI: {str(e)}[/red]")
        return False


def _install_kubectl(system_info) -> bool:
    """Instala kubectl baseado no sistema operacional."""
    try:
        from ..system.system_detector import OperatingSystem
        
        print(":gear: [blue]Instalando kubectl...[/blue]")
        
        if system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            # Ubuntu/Debian - via repositório oficial do Kubernetes
            # Limpar repositórios corrompidos primeiro
            _cleanup_corrupted_repositories()
            subprocess.run(["sudo", "apt-get", "update"], check=True, capture_output=True)
            subprocess.run([
                "sudo", "apt-get", "install", "-y", "ca-certificates", "curl", "apt-transport-https"
            ], check=True, capture_output=True)
            
            # Adicionar chave GPG do Kubernetes
            subprocess.run([
                "sudo", "mkdir", "-p", "/etc/apt/keyrings"
            ], capture_output=True)
            subprocess.run([
                "bash", "-c", 
                "curl -fsSL https://pkgs.k8s.io/core:/stable:/v1.28/deb/Release.key | sudo gpg --dearmor -o /etc/apt/keyrings/kubernetes-apt-keyring.gpg"
            ], check=True, capture_output=True)
            
            # Adicionar repositório
            subprocess.run([
                "bash", "-c",
                "echo 'deb [signed-by=/etc/apt/keyrings/kubernetes-apt-keyring.gpg] https://pkgs.k8s.io/core:/stable:/v1.28/deb/ /' | sudo tee /etc/apt/sources.list.d/kubernetes.list"
            ], check=True, capture_output=True)
            
            subprocess.run(["sudo", "apt-get", "update"], check=True, capture_output=True)
            subprocess.run(["sudo", "apt-get", "install", "-y", "kubectl"], check=True, capture_output=True)
            
        elif system_info.os_type == OperatingSystem.MACOS:
            # macOS - via Homebrew
            subprocess.run(["brew", "install", "kubectl"], check=True, capture_output=True)
            
        elif system_info.os_type in [OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA]:
            # CentOS/RHEL/Fedora - via repositório oficial
            repo_content = """[kubernetes]
name=Kubernetes
baseurl=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/
enabled=1
gpgcheck=1
gpgkey=https://pkgs.k8s.io/core:/stable:/v1.28/rpm/repodata/repomd.xml.key"""
            
            subprocess.run([
                "bash", "-c", f"echo '{repo_content}' | sudo tee /etc/yum.repos.d/kubernetes.repo"
            ], check=True)
            
            pkg_manager = "dnf" if system_info.os_type == OperatingSystem.FEDORA else "yum"
            subprocess.run(["sudo", pkg_manager, "install", "-y", "kubectl"], check=True, capture_output=True)
        
        else:
            print(":warning: [yellow]Sistema não suportado para kubectl[/yellow]")
            return False
            
        print(":white_check_mark: [green]kubectl instalado com sucesso![/green]")
        return True
        
    except subprocess.CalledProcessError as e:
        print(f":x: [red]Erro ao instalar kubectl: {e}[/red]")
        return False
    except Exception as e:
        print(f":x: [red]Erro inesperado ao instalar kubectl: {str(e)}[/red]")
        return False


def _install_ansible(system_info) -> bool:
    """Instala Ansible baseado no sistema operacional."""
    try:
        from ..system.system_detector import OperatingSystem
        
        print(":gear: [blue]Instalando Ansible...[/blue]")
        
        if system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            # Ubuntu/Debian - via pip (método mais confiável)
            # Limpar repositórios corrompidos primeiro
            _cleanup_corrupted_repositories()
            subprocess.run(["sudo", "apt-get", "update"], check=True, capture_output=True)
            subprocess.run(["sudo", "apt-get", "install", "-y", "python3-pip"], check=True, capture_output=True)
            
            # Instalar Ansible globalmente para que fique disponível no PATH
            subprocess.run(["sudo", "pip3", "install", "ansible"], check=True, capture_output=True)
            
            # Verificar se o binário está acessível e criar link se necessário
            try:
                subprocess.run(["ansible", "--version"], check=True, capture_output=True)
            except (FileNotFoundError, subprocess.CalledProcessError):
                # Se não encontrar, tentar criar link simbólico
                ansible_paths = [
                    "/usr/local/bin/ansible",
                    "/home/user/.local/bin/ansible",
                    "/usr/bin/ansible"
                ]
                for path in ansible_paths:
                    if subprocess.run(["test", "-f", path], capture_output=True).returncode == 0:
                        subprocess.run(["sudo", "ln", "-sf", path, "/usr/bin/ansible"], capture_output=True)
                        break
            
        elif system_info.os_type == OperatingSystem.MACOS:
            # macOS - via Homebrew
            subprocess.run(["brew", "install", "ansible"], check=True, capture_output=True)
            
        elif system_info.os_type in [OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA]:
            # CentOS/RHEL/Fedora - via pip
            pkg_manager = "dnf" if system_info.os_type == OperatingSystem.FEDORA else "yum"
            subprocess.run(["sudo", pkg_manager, "install", "-y", "python3-pip"], check=True, capture_output=True)
            
            # Instalar Ansible globalmente
            subprocess.run(["sudo", "pip3", "install", "ansible"], check=True, capture_output=True)
            
            # Verificar se o binário está acessível
            try:
                subprocess.run(["ansible", "--version"], check=True, capture_output=True)
            except (FileNotFoundError, subprocess.CalledProcessError):
                # Se não encontrar, tentar criar link simbólico
                ansible_paths = [
                    "/usr/local/bin/ansible",
                    "/home/user/.local/bin/ansible",
                    "/usr/bin/ansible"
                ]
                for path in ansible_paths:
                    if subprocess.run(["test", "-f", path], capture_output=True).returncode == 0:
                        subprocess.run(["sudo", "ln", "-sf", path, "/usr/bin/ansible"], capture_output=True)
                        break
        
        else:
            print(":warning: [yellow]Sistema não suportado para Ansible[/yellow]")
            return False
            
        print(":white_check_mark: [green]Ansible instalado com sucesso![/green]")
        return True
        
    except subprocess.CalledProcessError as e:
        print(f":x: [red]Erro ao instalar Ansible: {e}[/red]")
        return False
    except Exception as e:
        print(f":x: [red]Erro inesperado ao instalar Ansible: {str(e)}[/red]")
        return False


def _install_watch(system_info) -> bool:
    """Instala watch baseado no sistema operacional."""
    try:
        from ..system.system_detector import OperatingSystem
        
        print(":gear: [blue]Instalando watch...[/blue]")
        
        if system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            # Ubuntu/Debian - via apt
            # Limpar repositórios corrompidos primeiro
            _cleanup_corrupted_repositories()
            subprocess.run(["sudo", "apt-get", "update"], check=True, capture_output=True)
            subprocess.run(["sudo", "apt-get", "install", "-y", "procps"], check=True, capture_output=True)
            
        elif system_info.os_type == OperatingSystem.MACOS:
            # macOS - via Homebrew
            subprocess.run(["brew", "install", "watch"], check=True, capture_output=True)
            
        elif system_info.os_type in [OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA]:
            # CentOS/RHEL/Fedora - via yum/dnf
            pkg_manager = "dnf" if system_info.os_type == OperatingSystem.FEDORA else "yum"
            subprocess.run(["sudo", pkg_manager, "install", "-y", "procps-ng"], check=True, capture_output=True)
        
        else:
            print(":warning: [yellow]Sistema não suportado para watch[/yellow]")
            return False
            
        print(":white_check_mark: [green]watch instalado com sucesso![/green]")
        return True
        
    except subprocess.CalledProcessError as e:
        print(f":x: [red]Erro ao instalar watch: {e}[/red]")
        return False
    except Exception as e:
        print(f":x: [red]Erro inesperado ao instalar watch: {str(e)}[/red]")
        return False


def _cleanup_corrupted_repositories() -> None:
    """Remove repositórios corrompidos que podem afetar apt-get update."""
    try:
        print(":broom: [blue]Limpando repositórios corrompidos...[/blue]")
        
        # Lista de arquivos de repositório que podem estar corrompidos
        corrupted_repos = [
            "/etc/apt/sources.list.d/hashicorp.list",
            "/etc/apt/sources.list.d/microsoft-prod.list",
            "/etc/apt/sources.list.d/azure-cli.list",
            "/etc/apt/sources.list.d/kubernetes.list"
        ]
        
        # Lista de chaves GPG que podem estar corrompidas
        corrupted_keys = [
            "/etc/apt/keyrings/hashicorp.gpg",
            "/etc/apt/keyrings/microsoft.gpg",
            "/etc/apt/keyrings/kubernetes-apt-keyring.gpg",
            "/etc/apt/trusted.gpg.d/microsoft.gpg"
        ]
        
        # Remover arquivos de repositório corrompidos
        for repo_file in corrupted_repos:
            subprocess.run([
                "sudo", "rm", "-f", repo_file
            ], capture_output=True)
        
        # Remover chaves GPG corrompidas
        for key_file in corrupted_keys:
            subprocess.run([
                "sudo", "rm", "-f", key_file
            ], capture_output=True)
        
        # Tentar atualizar repositórios para limpar cache
        result = subprocess.run([
            "sudo", "apt-get", "update"
        ], capture_output=True)
        
        if result.returncode == 0:
            print(":white_check_mark: [green]Repositórios limpos com sucesso[/green]")
        else:
            print(":warning: [yellow]Aviso: Alguns repositórios ainda podem ter problemas[/yellow]")
        
    except Exception as e:
        print(f":warning: [yellow]Aviso: Não foi possível limpar repositórios: {str(e)}[/yellow]")


"""Instalador do Azure CLI para diferentes sistemas operacionais."""

import subprocess
import tempfile
from pathlib import Path
from typing import Optional, List
from rich import print

from .base_installer import BaseInstaller
from ..system_detector import SystemInfo, OperatingSystem


class AzureCliInstaller(BaseInstaller):
    """Instalador especializado para Azure CLI."""
    
    def __init__(self, system_info: SystemInfo):
        """
        Inicializa o instalador do Azure CLI.
        
        Args:
            system_info: Informações do sistema operacional
        """
        super().__init__(system_info)
        self.tool_name = "Azure CLI"
        
    def is_installed(self) -> bool:
        """
        Verifica se o Azure CLI está instalado.
        
        Returns:
            bool: True se o Azure CLI estiver instalado
        """
        try:
            result = subprocess.run(
                ["az", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            return result.returncode == 0
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return False
    
    def get_installed_version(self) -> Optional[str]:
        """
        Obtém a versão instalada do Azure CLI.
        
        Returns:
            Optional[str]: Versão instalada ou None
        """
        try:
            result = subprocess.run(
                ["az", "--version"],
                capture_output=True,
                text=True,
                timeout=10
            )
            if result.returncode == 0:
                # O output é JSON-like, pegar a primeira linha que contém "azure-cli"
                for line in result.stdout.split('\n'):
                    if 'azure-cli' in line and 'core' not in line:
                        # azure-cli         2.53.0
                        parts = line.split()
                        if len(parts) >= 2:
                            return parts[1]
            return None
        except (FileNotFoundError, subprocess.TimeoutExpired):
            return None
    
    def install(self) -> bool:
        """
        Instala o Azure CLI baseado no sistema operacional.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(f":gear: [blue]Instalando {self.tool_name}...[/blue]")
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                return self._install_macos()
            
            elif self.system_info.os_type in [
                OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
                OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
            ]:
                return self._install_ubuntu()
            
            elif self.system_info.os_type in [
                OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
            ]:
                return self._install_redhat()
            
            else:
                print(f":warning: [yellow]Sistema {self.system_info.os_type.value} não suportado para instalação automática do Azure CLI[/yellow]")
                return self._show_manual_instructions()
        
        except Exception as e:
            print(f":x: [red]Erro durante instalação do Azure CLI: {str(e)}[/red]")
            return False
    
    def _install_macos(self) -> bool:
        """Instala Azure CLI no macOS."""
        print(":apple: [blue]Detectado macOS - tentando Homebrew primeiro[/blue]")
        
        # Verificar se Homebrew está disponível
        if self._check_homebrew():
            try:
                print(":beer: [blue]Instalando Azure CLI via Homebrew...[/blue]")
                result = subprocess.run([
                    "brew", "install", "azure-cli"
                ], capture_output=True, text=True, timeout=300)
                
                if result.returncode == 0:
                    print(":white_check_mark: [green]Azure CLI instalado via Homebrew![/green]")
                    return True
                else:
                    print(f":warning: [yellow]Falha no Homebrew: {result.stderr}[/yellow]")
            except subprocess.TimeoutExpired:
                print(":warning: [yellow]Timeout do Homebrew[/yellow]")
        
        # Fallback para script oficial
        print(":information: [blue]Tentando instalação via script oficial...[/blue]")
        return self._install_via_curl_script()
    
    def _install_ubuntu(self) -> bool:
        """Instala Azure CLI no Ubuntu/Debian."""
        print(":gear: [blue]Detectado Ubuntu/Debian - usando repositório oficial[/blue]")
        
        try:
            # Método 1: Tentar repositório oficial Microsoft
            if self._install_ubuntu_repo():
                return True
            
            # Método 2: Fallback para script de instalação
            print(":information: [blue]Tentando instalação via script oficial...[/blue]")
            return self._install_via_curl_script()
        
        except Exception as e:
            print(f":warning: [yellow]Erro no método de repositório: {str(e)}[/yellow]")
            return self._install_via_curl_script()
    
    def _install_ubuntu_repo(self) -> bool:
        """Instala via repositório oficial da Microsoft."""
        try:
            # Instalar dependências
            print(":package: [blue]Instalando dependências...[/blue]")
            
            # Limpar repositórios corrompidos antes de tentar atualizar
            self._cleanup_corrupted_repositories()
            
            subprocess.run([
                "sudo", "apt-get", "update"
            ], check=True, capture_output=True)
            
            subprocess.run([
                "sudo", "apt-get", "install", "-y", "ca-certificates", "curl", "apt-transport-https", "lsb-release", "gnupg"
            ], check=True, capture_output=True)
            
            # Adicionar chave GPG da Microsoft
            print(":key: [blue]Adicionando chave GPG da Microsoft...[/blue]")
            with tempfile.NamedTemporaryFile(mode='w', suffix='.sh', delete=False) as f:
                f.write("""#!/bin/bash
# Criar diretório de keyrings se não existir
sudo mkdir -p /etc/apt/keyrings
curl -sLS https://packages.microsoft.com/keys/microsoft.asc | gpg --dearmor | sudo tee /etc/apt/keyrings/microsoft.gpg > /dev/null
sudo chmod go+r /etc/apt/keyrings/microsoft.gpg
""")
                script_path = f.name
            
            subprocess.run(["bash", script_path], check=True)
            Path(script_path).unlink()
            
            # Adicionar repositório
            print(":package: [blue]Adicionando repositório Microsoft...[/blue]")
            distro = self._get_ubuntu_codename()
            repo_line = f"deb [arch=amd64,arm64,armhf signed-by=/etc/apt/keyrings/microsoft.gpg] https://packages.microsoft.com/repos/azure-cli/ {distro} main"
            
            subprocess.run([
                "bash", "-c", f"echo '{repo_line}' | sudo tee /etc/apt/sources.list.d/azure-cli.list"
            ], check=True)
            
            # Atualizar e instalar
            print(":arrows_counterclockwise: [blue]Atualizando repositórios...[/blue]")
            subprocess.run(["sudo", "apt-get", "update"], check=True, capture_output=True)
            
            print(":package: [blue]Instalando Azure CLI...[/blue]")
            subprocess.run(["sudo", "apt-get", "install", "-y", "azure-cli"], check=True)
            
            print(":white_check_mark: [green]Azure CLI instalado via repositório oficial![/green]")
            return True
        
        except subprocess.CalledProcessError as e:
            print(f":warning: [yellow]Falha no repositório oficial: {e}[/yellow]")
            return False
    
    def _install_redhat(self) -> bool:
        """Instala Azure CLI no CentOS/RHEL/Fedora."""
        print(":gear: [blue]Detectado sistema RedHat - usando repositório oficial[/blue]")
        
        try:
            # Determinar gerenciador de pacotes
            if self.system_info.os_type == OperatingSystem.FEDORA:
                pkg_manager = "dnf"
            else:
                pkg_manager = "yum"
            
            # Importar chave GPG da Microsoft
            print(":key: [blue]Importando chave GPG da Microsoft...[/blue]")
            subprocess.run([
                "sudo", "rpm", "--import", "https://packages.microsoft.com/keys/microsoft.asc"
            ], check=True)
            
            # Adicionar repositório
            print(":package: [blue]Adicionando repositório Microsoft...[/blue]")
            repo_content = """[azure-cli]
name=Azure CLI
baseurl=https://packages.microsoft.com/yumrepos/azure-cli
enabled=1
gpgcheck=1
gpgkey=https://packages.microsoft.com/keys/microsoft.asc"""
            
            subprocess.run([
                "bash", "-c", f"echo '{repo_content}' | sudo tee /etc/yum.repos.d/azure-cli.repo"
            ], check=True)
            
            # Instalar Azure CLI
            print(":package: [blue]Instalando Azure CLI...[/blue]")
            subprocess.run([
                "sudo", pkg_manager, "install", "-y", "azure-cli"
            ], check=True)
            
            print(":white_check_mark: [green]Azure CLI instalado via repositório oficial![/green]")
            return True
        
        except subprocess.CalledProcessError as e:
            print(f":warning: [yellow]Falha no repositório: {e}[/yellow]")
            return self._install_via_curl_script()
    
    def _install_via_curl_script(self) -> bool:
        """Instala Azure CLI via script oficial (método universal)."""
        print(":globe_with_meridians: [blue]Instalando via script oficial da Microsoft...[/blue]")
        
        try:
            result = subprocess.run([
                "curl", "-sL", "https://aka.ms/InstallAzureCLIDeb"
            ], capture_output=True, text=True, timeout=60)
            
            if result.returncode != 0:
                print(":x: [red]Falha ao baixar script de instalação[/red]")
                return False
            
            # Executar script
            process = subprocess.run([
                "sudo", "bash"
            ], input=result.stdout, text=True, capture_output=True, timeout=300)
            
            if process.returncode == 0:
                print(":white_check_mark: [green]Azure CLI instalado via script oficial![/green]")
                return True
            else:
                print(f":x: [red]Falha na execução do script: {process.stderr}[/red]")
                return False
        
        except subprocess.TimeoutExpired:
            print(":x: [red]Timeout durante instalação via script[/red]")
            return False
        except Exception as e:
            print(f":x: [red]Erro na instalação via script: {str(e)}[/red]")
            return False
    
    def _get_ubuntu_codename(self) -> str:
        """Obtém o codename da distribuição Ubuntu."""
        try:
            result = subprocess.run([
                "lsb_release", "-cs"
            ], capture_output=True, text=True)
            
            if result.returncode == 0:
                return result.stdout.strip()
        except:
            pass
        
        # Fallback baseado na versão
        version_map = {
            "22.04": "jammy",
            "20.04": "focal", 
            "18.04": "bionic"
        }
        return version_map.get(self.system_info.os_version, "focal")
    
    def _check_homebrew(self) -> bool:
        """Verifica se o Homebrew está instalado."""
        try:
            subprocess.run(["brew", "--version"], capture_output=True, check=True)
            return True
        except (FileNotFoundError, subprocess.CalledProcessError):
            return False
    
    def _show_manual_instructions(self) -> bool:
        """Mostra instruções para instalação manual."""
        print(":information_source: [cyan]Instruções para instalação manual do Azure CLI:[/cyan]")
        print()
        print("1. Acesse: https://docs.microsoft.com/en-us/cli/azure/install-azure-cli")
        print("2. Escolha o método de instalação para seu sistema operacional")
        print("3. Execute os comandos fornecidos na documentação")
        print("4. Verifique a instalação com: az --version")
        print()
        return False
    
    def get_install_commands(self) -> List[str]:
        """
        Retorna lista de comandos para instalação manual.
        
        Returns:
            List[str]: Lista de comandos
        """
        if self.system_info.os_type == OperatingSystem.MACOS:
            return [
                "# Via Homebrew (recomendado)",
                "brew install azure-cli",
                "",
                "# Ou via script oficial:",
                "curl -sL https://aka.ms/InstallAzureCLIDeb | sudo bash"
            ]
        
        elif self.system_info.os_type in [
            OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
            OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
        ]:
            return [
                "# Via repositório oficial Microsoft",
                "sudo apt-get update",
                "sudo apt-get install ca-certificates curl apt-transport-https lsb-release gnupg",
                "curl -sLS https://packages.microsoft.com/keys/microsoft.asc | gpg --dearmor | sudo tee /etc/apt/keyrings/microsoft.gpg > /dev/null",
                "sudo chmod go+r /etc/apt/keyrings/microsoft.gpg",
                f"echo 'deb [arch=amd64,arm64,armhf signed-by=/etc/apt/keyrings/microsoft.gpg] https://packages.microsoft.com/repos/azure-cli/ {self._get_ubuntu_codename()} main' | sudo tee /etc/apt/sources.list.d/azure-cli.list",
                "sudo apt-get update",
                "sudo apt-get install azure-cli"
            ]
        
        elif self.system_info.os_type in [
            OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
        ]:
            pkg_manager = "dnf" if self.system_info.os_type == OperatingSystem.FEDORA else "yum"
            return [
                "# Via repositório oficial Microsoft",
                "sudo rpm --import https://packages.microsoft.com/keys/microsoft.asc",
                "echo -e '[azure-cli]\\nname=Azure CLI\\nbaseurl=https://packages.microsoft.com/yumrepos/azure-cli\\nenabled=1\\ngpgcheck=1\\ngpgkey=https://packages.microsoft.com/keys/microsoft.asc' | sudo tee /etc/yum.repos.d/azure-cli.repo",
                f"sudo {pkg_manager} install azure-cli"
            ]
        
        else:
            return [
                "# Instalação manual",
                "1. Acesse: https://docs.microsoft.com/en-us/cli/azure/install-azure-cli",
                "2. Escolha o método de instalação para seu sistema operacional",
                "3. Execute os comandos fornecidos na documentação",
                "4. Verifique com: az --version"
            ]

    def uninstall(self) -> bool:
        """
        Remove o Azure CLI do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        print(f":wastebasket: [blue]Removendo {self.tool_name}...[/blue]")
        
        try:
            if self.system_info.os_type == OperatingSystem.MACOS:
                # Tentar remover via Homebrew primeiro
                try:
                    subprocess.run(["brew", "uninstall", "azure-cli"], check=True)
                    print(":white_check_mark: [green]Azure CLI removido via Homebrew![/green]")
                    return True
                except subprocess.CalledProcessError:
                    pass
            
            elif self.system_info.os_type in [
                OperatingSystem.UBUNTU, OperatingSystem.WSL_UBUNTU,
                OperatingSystem.DEBIAN, OperatingSystem.WSL_DEBIAN
            ]:
                try:
                    subprocess.run(["sudo", "apt-get", "remove", "-y", "azure-cli"], check=True)
                    print(":white_check_mark: [green]Azure CLI removido via apt![/green]")
                    return True
                except subprocess.CalledProcessError:
                    pass
            
            elif self.system_info.os_type in [
                OperatingSystem.CENTOS, OperatingSystem.RHEL, OperatingSystem.FEDORA
            ]:
                pkg_manager = "dnf" if self.system_info.os_type == OperatingSystem.FEDORA else "yum"
                try:
                    subprocess.run(["sudo", pkg_manager, "remove", "-y", "azure-cli"], check=True)
                    print(f":white_check_mark: [green]Azure CLI removido via {pkg_manager}![/green]")
                    return True
                except subprocess.CalledProcessError:
                    pass
            
            print(":x: [red]Azure CLI não encontrado para remoção[/red]")
            return False
        
        except Exception as e:
            print(f":x: [red]Erro ao remover Azure CLI: {str(e)}[/red]")
            return False
    
    def _cleanup_corrupted_repositories(self) -> None:
        """Remove repositórios corrompidos que podem afetar apt-get update."""
        try:
            print(":broom: [blue]Limpando repositórios corrompidos...[/blue]")
            
            # Lista de arquivos de repositório que podem estar corrompidos
            corrupted_repos = [
                "/etc/apt/sources.list.d/hashicorp.list",
                "/etc/apt/sources.list.d/microsoft-prod.list",
                "/etc/apt/sources.list.d/azure-cli.list"
            ]
            
            # Lista de chaves GPG que podem estar corrompidas
            corrupted_keys = [
                "/etc/apt/keyrings/hashicorp.gpg",
                "/etc/apt/keyrings/microsoft.gpg",
                "/etc/apt/trusted.gpg.d/microsoft.gpg"
            ]
            
            # Remover arquivos de repositório corrompidos
            for repo_file in corrupted_repos:
                subprocess.run([
                    "sudo", "rm", "-f", repo_file
                ], capture_output=True)
            
            # Remover chaves GPG corrompidas
            for key_file in corrupted_keys:
                subprocess.run([
                    "sudo", "rm", "-f", key_file
                ], capture_output=True)
            
            # Tentar atualizar repositórios para limpar cache
            result = subprocess.run([
                "sudo", "apt-get", "update"
            ], capture_output=True)
            
            if result.returncode == 0:
                print(":white_check_mark: [green]Repositórios limpos com sucesso[/green]")
            else:
                print(":warning: [yellow]Aviso: Alguns repositórios ainda podem ter problemas[/yellow]")
            
        except Exception as e:
            print(f":warning: [yellow]Aviso: Não foi possível limpar repositórios: {str(e)}[/yellow]")


"""Constantes e configurações da aplicação."""

import enum
from pathlib import Path




class Tool(str, enum.Enum):
    """Ferramentas que podem ser instaladas."""
    DOCKER = "docker"
    TERRAFORM = "terraform"
    ANSIBLE = "ansible"
    AZURE_CLI = "az"
    AWS_CLI = "aws"
    GIT = "git"
    KUBECTL = "kubectl"
    WATCH = "watch"


# Configurações de paths
BASE_PATH = Path(__file__).parent.parent.parent
SRC_PATH = BASE_PATH / "src"

# Configurações das ferramentas DevOps
DEVOPS_TOOLS_CONFIG = {
    Tool.DOCKER: {
        "name": "Docker",
        "description": "Plataforma de containerização",
        "check_command": ["docker", "--version"],
        "priority": 1,
        "required": False
    },
    Tool.TERRAFORM: {
        "name": "Terraform",
        "description": "Ferramenta de infraestrutura como código",
        "check_command": ["terraform", "--version"],
        "priority": 2,
        "required": False
    },
    Tool.GIT: {
        "name": "Git",
        "description": "Sistema de controle de versão",
        "check_command": ["git", "--version"],
        "priority": 3,
        "required": False
    },
    Tool.AZURE_CLI: {
        "name": "Azure CLI",
        "description": "Interface de linha de comando da Azure",
        "check_command": ["az", "--version"],
        "priority": 4,
        "required": False
    },
    Tool.AWS_CLI: {
        "name": "AWS CLI v2",
        "description": "Interface de linha de comando da AWS",
        "check_command": ["aws", "--version"],
        "priority": 5,
        "required": False
    },
    Tool.KUBECTL: {
        "name": "kubectl",
        "description": "Cliente para Kubernetes",
        "check_command": ["kubectl", "version", "--client"],
        "priority": 6,
        "required": False
    },
    Tool.ANSIBLE: {
        "name": "Ansible",
        "description": "Automação e gerenciamento de configuração",
        "check_command": ["ansible", "--version"],
        "priority": 7,
        "required": False
    },
    Tool.WATCH: {
        "name": "watch",
        "description": "Executa comandos periodicamente",
        "check_command": ["watch", "--version"],
        "priority": 8,
        "required": False
    }
}


"""Instalador do Docker para Ubuntu/Debian."""

import subprocess
from typing import List
from rich import print

from .base_installer import BaseInstaller


class UbuntuInstaller(BaseInstaller):
    """Instalador do Docker para Ubuntu e Debian."""
    
    def install(self) -> bool:
        """
        Instala o Docker no Ubuntu/Debian.
        
        Returns:
            bool: True se a instalação foi bem-sucedida
        """
        print(f":package: Instalando Docker no {self.system_info.os_type.value}...")
        
        try:
            # 1. Atualizar repositórios
            print("  [blue]1/6[/blue] Atualizando repositórios...")
            self._run_command(["sudo", "apt", "update"])
            
            # 2. Instalar dependências
            print("  [blue]2/6[/blue] Instalando dependências...")
            dependencies = [
                "apt-transport-https",
                "ca-certificates", 
                "curl",
                "gnupg",
                "lsb-release"
            ]
            cmd = ["sudo", "apt", "install", "-y"] + dependencies
            self._run_command(cmd)
            
            # 3. Adicionar chave GPG oficial do Docker
            print("  [blue]3/6[/blue] Adicionando chave GPG do Docker...")
            self._run_command([
                "curl", "-fsSL", "https://download.docker.com/linux/ubuntu/gpg",
                "|", "sudo", "gpg", "--dearmor", "-o", "/usr/share/keyrings/docker-archive-keyring.gpg"
            ], shell=True)
            
            # 4. Adicionar repositório do Docker
            print("  [blue]4/6[/blue] Adicionando repositório do Docker...")
            distro = "ubuntu" if "ubuntu" in self.system_info.os_type.value else "debian"
            repo_cmd = [
                "echo",
                f'"deb [arch=amd64 signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/{distro} $(lsb_release -cs) stable"',
                "|", "sudo", "tee", "/etc/apt/sources.list.d/docker.list", ">", "/dev/null"
            ]
            self._run_command(repo_cmd, shell=True)
            
            # 5. Atualizar repositórios novamente
            print("  [blue]5/6[/blue] Atualizando repositórios com Docker...")
            self._run_command(["sudo", "apt", "update"])
            
            # 6. Instalar Docker
            print("  [blue]6/6[/blue] Instalando Docker CE...")
            self._run_command([
                "sudo", "apt", "install", "-y", 
                "docker-ce", "docker-ce-cli", "containerd.io"
            ])
            
            # Configurar Docker para usuário atual
            self._configure_docker_user()
            
            print("  [green]✓[/green] Docker instalado com sucesso!")
            return True
            
        except subprocess.CalledProcessError as e:
            print(f"  [red]✗[/red] Erro durante a instalação: {e}")
            return False
        except Exception as e:
            print(f"  [red]✗[/red] Erro inesperado: {e}")
            return False
    
    def _configure_docker_user(self) -> None:
        """Configura o Docker para o usuário atual."""
        try:
            print("  [yellow]📝[/yellow] Configurando permissões do Docker...")
            
            # Adicionar usuário ao grupo docker
            import os
            username = os.getenv("USER")
            if username:
                self._run_command(["sudo", "usermod", "-aG", "docker", username])
                print(f"  [green]✓[/green] Usuário {username} adicionado ao grupo docker")
                print("  [yellow]⚠[/yellow] Faça logout/login para aplicar as permissões")
            
            # Iniciar serviço Docker
            self._run_command(["sudo", "systemctl", "enable", "docker"])
            self._run_command(["sudo", "systemctl", "start", "docker"])
            
        except Exception as e:
            print(f"  [yellow]![/yellow] Aviso: Erro na configuração de usuário: {e}")
    
    def uninstall(self) -> bool:
        """
        Remove o Docker do sistema.
        
        Returns:
            bool: True se a remoção foi bem-sucedida
        """
        try:
            print(":wastebasket: Removendo Docker...")
            
            # Parar serviços
            self._run_command(["sudo", "systemctl", "stop", "docker"], ignore_errors=True)
            self._run_command(["sudo", "systemctl", "disable", "docker"], ignore_errors=True)
            
            # Remover pacotes
            self._run_command([
                "sudo", "apt", "remove", "-y",
                "docker-ce", "docker-ce-cli", "containerd.io"
            ])
            
            # Remover repositório (opcional)
            self._run_command([
                "sudo", "rm", "-f", "/etc/apt/sources.list.d/docker.list"
            ], ignore_errors=True)
            
            print("  [green]✓[/green] Docker removido com sucesso!")
            return True
            
        except Exception as e:
            print(f"  [red]✗[/red] Erro durante a remoção: {e}")
            return False
    
    def get_install_commands(self) -> List[str]:
        """Retorna lista de comandos para instalação manual."""
        return [
            "sudo apt update",
            "sudo apt install -y apt-transport-https ca-certificates curl gnupg lsb-release",
            "curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo gpg --dearmor -o /usr/share/keyrings/docker-archive-keyring.gpg",
            'echo "deb [arch=amd64 signed-by=/usr/share/keyrings/docker-archive-keyring.gpg] https://download.docker.com/linux/ubuntu $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list > /dev/null',
            "sudo apt update", 
            "sudo apt install -y docker-ce docker-ce-cli containerd.io",
            "sudo usermod -aG docker $USER",
            "sudo systemctl enable docker",
            "sudo systemctl start docker"
        ]